/*******************************************************************************
* Accel  - Function Set fo Accel Example
* ------------------------------------------------------------------------------
* Example reads accel values and displays graph on TFT for all three axes
*******************************************************************************/
#include "built_in.h"
#include "mikromedia_5_TIVA_objects.h"
#include "ADXL345_driver.h"

//  static
// we use Scroll feature to display graph
static const _SCROLL_ACCEL_FIRST_LINE =  50;
static const _SCROLL_ACCEL_LAST_LINE  = 440;
static const _SCROLL_ACCEL_HEIGHT     = 390;

static int readings[3] = {0, 0, 0}; // X, Y and Z buffer

typedef struct Accel_values{
  int Xvalue;
  int Yvalue;
  int Zvalue;
} TAccel_Values;

static TAccel_values Accel_vals, Old_Accel_vals = {0, 0, 0};
/******************************************************************************/
static int _disp_accel_scroll;
static const _ACCEL_UP = 1, _ACCEL_DOWN = 0;
static int Accel_counter;

/*******************************************************************************
* Function Accel_Average()
* ------------------------------------------------------------------------------
* Overview: Function calculate the average values of the X, Y and Z axis reads
* Input: Nothing
* Output: X,Y and Z values are stored in readings[] buffer
*******************************************************************************/
static void Accel_Average() {
  int i, sx, sy, sz, xx, yy, zz;

  // sum
  sx = sy = sz = 0;

  // average accelerometer reading over last 16 samples
  for (i=0; i<16; i++) {
    ADXL345_Read(&xx, &yy, &zz);
    sx += xx;
    sy += yy;
    sz += zz;
  }
  // average
  readings[0] = sx / 16;
  readings[1] = sy / 16;
  readings[2] = sz / 16;
}

/*******************************************************************************
* Function Display_Value(TLabel *label, int val)
* ------------------------------------------------------------------------------
* Overview: Function display accel values
* Input: desired pointer to Label, measured value
* Output: Nothing
*******************************************************************************/
static void Display_Value(TLabel *label, int val) {
char text[7];
char text1[10];
char *ptr;
  ptr = label->Caption;
  TFTIntern_Set_Pen(Accelerometer.Color,  0);
  TFTIntern_Set_Brush(1, Accelerometer.Color, 0, 0, 0, 0);
  IntToStr(val & 0xFFFC, text);
  strcpy(text1, ptr);
  strcat(text1, text);
  TFTIntern_Rectangle(label->Left, Label->Top, label->Left + 100, Label->Top + 30);
  label->Caption = text1;
  Drawlabel(label);
  label->Caption = ptr;
}

/*******************************************************************************
* Function Scroll_ACCEL(char Adir)
* ------------------------------------------------------------------------------
* Overview: Function scrolls graph
* Input: Scroll direction
* Output: Nothing
*******************************************************************************/
static void Scroll_ACCEL(char Adir) {

if (Adir == _ACCEL_UP) {
      if (_disp_accel_scroll != 0)
        _disp_accel_scroll--;
      else {
        _disp_accel_scroll = _SCROLL_ACCEL_HEIGHT - 1;
      }
  }
  else {
      if (_disp_accel_scroll != _SCROLL_ACCEL_HEIGHT - 1)
        _disp_accel_scroll++;
      else {
        _disp_accel_scroll = 0;
      }
  }
}

/*******************************************************************************
* Function WriteGraph(TAccel_values *old, TAccel_values *new)
* ------------------------------------------------------------------------------
* Overview: Function writes new graph values on display
* Input: accel values
* Output: Nothing
*******************************************************************************/
static void WriteGraph(TAccel_values *old, TAccel_values *new){
int temp1, temp2;

  // Delete one row with old values
  TFTIntern_Set_Pen(Box1.COlor, 1);
  TFTIntern_H_Line(0, 800, _disp_accel_scroll + _SCROLL_ACCEL_FIRST_LINE);

  // Draw current accel value for x-axis
  TFTIntern_Set_Pen(CL_RED_Intern, 2);
  temp1 = ((unsigned long)old->Xvalue * 80) / 256 + 200;
  temp2 = ((unsigned long)new->Xvalue * 80) / 256 + 200;
  TFTIntern_Line(temp1, _disp_accel_scroll + _SCROLL_ACCEL_FIRST_LINE, temp2, _disp_accel_scroll + 1 + _SCROLL_ACCEL_FIRST_LINE);

  // Draw current accel value for y-axis
  TFTIntern_Set_Pen(CL_BLUE_Intern, 2);
  temp1 = ((unsigned long)old->Yvalue * 80) / 256 + 400;
  temp2 = ((unsigned long)new->Yvalue * 80) / 256 + 400;
  TFTIntern_Line(temp1, _disp_accel_scroll + _SCROLL_ACCEL_FIRST_LINE, temp2, _disp_accel_scroll + 1 + _SCROLL_ACCEL_FIRST_LINE);

  // Draw current accel value for z-axis
  TFTIntern_Set_Pen(CL_GREEN_Intern, 2);
  temp1 = ((unsigned long)old->Zvalue * 80) / 256 + 600;
  temp2 = ((unsigned long)new->Zvalue * 80) / 256 + 600;
  TFTIntern_Line(temp1, _disp_accel_scroll + _SCROLL_ACCEL_FIRST_LINE, temp2, _disp_accel_scroll + 1 + _SCROLL_ACCEL_FIRST_LINE);

  // Draw Vertical axis
  TFTIntern_Dot(200, _disp_accel_scroll + _SCROLL_ACCEL_FIRST_LINE, CL_GRAY_Intern);
  TFTIntern_Dot(400, _disp_accel_scroll + _SCROLL_ACCEL_FIRST_LINE, CL_GRAY_Intern);
  TFTIntern_Dot(600, _disp_accel_scroll + _SCROLL_ACCEL_FIRST_LINE, CL_GRAY_Intern);
  
  TFTIntern_Set_Pen(Box1.COlor, 2);
  if (_disp_accel_scroll + _SCROLL_ACCEL_FIRST_LINE + 5 < _SCROLL_ACCEL_LAST_LINE){
    TFTIntern_H_Line(0, 800, _disp_accel_scroll + _SCROLL_ACCEL_FIRST_LINE + 2);
    TFTIntern_H_Line(0, 800, _disp_accel_scroll + _SCROLL_ACCEL_FIRST_LINE + 3);
    TFTIntern_H_Line(0, 800, _disp_accel_scroll + _SCROLL_ACCEL_FIRST_LINE + 4);
    TFTIntern_Set_Pen(CL_GRAY_Intern, 2);
    TFTIntern_H_Line(0, 800, _disp_accel_scroll + _SCROLL_ACCEL_FIRST_LINE + 5);
  }
}

/*******************************************************************************
* Function ACCEL_Start()
* ------------------------------------------------------------------------------
* Overview: Function Initialize I2C bus and accel module
* Input: Nothing
* Output: test status: 0 - skiped; 1 - pass; 2 - fail
*******************************************************************************/
void ACCEL_Start() {
  // Reset error flag
  _disp_accel_scroll = 0;
  Accel_counter = 0;

  // Initialize I2C communication
  I2C2_Init_Advanced(400000, &_GPIO_MODULE_I2C2_P56);
  Delay_ms(100);
  // Initialize ADXL345 accelerometer
  if (ADXL345_Init() == 0) {
    Delay_ms(100);
  }
}

/*******************************************************************************
* Function ACCEL_Test()
* ------------------------------------------------------------------------------
* Overview: Function reads accel values
* Input: Nothing
* Output: Nothing
*******************************************************************************/

static void ACCEL_Test(TAccel_values *values) {
  Accel_Average();               // Calculate average X, Y and Z reads
  
  values->Xvalue = readings[0];
  values->Yvalue = readings[1];
  values->Zvalue = readings[2];
}

/*******************************************************************************
* Function doAccel()
* ------------------------------------------------------------------------------
* Overview: Function run the accel test, should be calles frequently.
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void doAccel(){
  
  if ((Accel_counter++) % 100 == 0){
    ACCEL_Test(&Accel_vals);
    Scroll_ACCEL(_ACCEL_Down);
    WriteGraph(&Old_Accel_vals, &Accel_vals);
    Old_accel_vals = Accel_vals;

    if (Accel_counter > 2000){
      Display_Value(&LAbel2, Accel_vals.Xvalue);
      Display_Value(&LAbel3, Accel_vals.Yvalue);
      Display_Value(&LAbel4, Accel_vals.Zvalue);
      Accel_counter = 0;
    }
  }
}

/*******************************************************************************
* End of File
*******************************************************************************/