/*******************************************************************************
* HID Mouse  - Function Set fo HID Mouse
* ------------------------------------------------------------------------------
* Example uses mouse pointer for coordination
*******************************************************************************/
#include <stdint.h>
#include "mikromedia_5_TIVA_resources.h"
#include "mikromedia_5_TIVA_objects.h"
#include "Init_routines.h"

// globals
char My_PenDown;

extern unsigned int Xcoord, Ycoord;

static const int16_t _X_MAX = 800;
static const int16_t _X_MIN = 0;
static const int16_t _Y_MAX = 480;
static const int16_t _Y_MIN = 0;

static const int8_t _MOUSE_SIZE = 20;
static const int16_t _DELTA = 50;

static unsigned int pointer_buffer[_MOUSE_SIZE * _MOUSE_SIZE];
static unsigned int pointer[_MOUSE_SIZE * _MOUSE_SIZE];

static int16_t cX, cY, pX, pY, oldx, oldy, cButton;
static char update = 0;
static char HID_Mouse_Connected = 0;
static char HID_Mouse_flag = 0;

// library requirements
typedef struct{
  void(*DeviceInit)();
  void(*ReportReceived)(uint8_t* rptData);
} TUSBHostHIDHandler;

void MouseInit();
void MouseReportReceived(uint8_t* reportData);
TUSBHostHIDHandler USBHost_HIDHandler = {
  MouseInit,
  MouseReportReceived
};
TUSBHostHIDHandler* USBHost_HIDHandlerPtr = &USBHost_HIDHandler;

typedef struct{
  int16_t button;
  int16_t x;
  int16_t y;
} TUSBHostHIDMouseReport;
TUSBHostHIDMouseReport mouseData;

/*******************************************************************************
* Function Copy_DisplayUnderPointer(unsigned int x, unsigned int y)
* ------------------------------------------------------------------------------
* Overview: Function copies display content at cursor location into buffer
* Input: desired coordinates of the cursor
* Output: Nothing
*******************************************************************************/
static void Copy_DisplayUnderPointer(unsigned int x, unsigned int y){
  unsigned int *ptr1, *ptr2;
  char j, i;
  unsigned long offset;
  
  ptr2 = pointer_buffer;

  for (j = 0; j < 20; j ++){
    offset = Get_Cursor_Offset(x, y + j);
    TFTIntern_Move_Cursor(x, y + j);
    ptr1 = My_MemoryStartPtr + offset;
    for (i = 0; i < 20; i ++)
      *ptr2++ = *ptr1--;
  }
}

/*******************************************************************************
* Function Copy_Buffer_ToDisplay(unsigned int x, unsigned int y)
* ------------------------------------------------------------------------------
* Overview: Function copies buffer content to display
* Input: desired coordinates of the cursor
* Output: Nothing
*******************************************************************************/
static void Copy_Buffer_ToDisplay(unsigned int x, unsigned int y){
  unsigned int *ptr1, *ptr2;
  char j;
  unsigned long offset;

  ptr1 = pointer_buffer;

  for (j = 0; j < 20; j ++){
    offset = Get_Cursor_Offset(x, y + j);
    TFTIntern_Move_Cursor(x, y + j);
    ptr2 = My_MemoryStartPtr + offset;
    EPI_WorkaroundHWordWriteArrayDec(ptr2, ptr1, 20);
    ptr1 += 20;
  }
}

/*******************************************************************************
* Function Copy_PointerToDisplay(unsigned int x, unsigned int y)
* ------------------------------------------------------------------------------
* Overview: Function displays pointer image on the display
* Input: desired coordinates of the cursor
* Output: Nothing
*******************************************************************************/
static void Copy_PointerToDisplay(unsigned int x, unsigned int y){
  unsigned int *ptr1, *ptr2, *ptr3;
  char j, i;
  unsigned long offset;

  ptr1 = pointer;
  ptr2 = pointer_buffer;

  for (j = 0; j < 20; J ++){
    for (i = 0; i < 20; i ++){
      if (*ptr1 != 0x07E0)
        TFTIntern_rowData[i] = *ptr1;
      else
        TFTIntern_rowData[i] = *ptr2;
      ptr1++;
      ptr2++;
    }

    offset = Get_Cursor_Offset(x, y + j);
    TFTIntern_Move_Cursor(x, y + j);
    ptr3 = My_MemoryStartPtr + offset;
    EPI_WorkaroundHWordWriteArrayDec(ptr3, TFTIntern_rowData, 20);
  }
}

/*******************************************************************************
* Function HID_Mouse_Restore_Display()
* ------------------------------------------------------------------------------
* Overview: Function restores display at current cursor location
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void HID_Mouse_Restore_Display(){
  if (HID_Mouse_Connected){
    Copy_Buffer_ToDisplay(oldx, oldy);
    update = 2;
    mouseData.button = 0;
  }
}

/*******************************************************************************
* Function HID_Mouse_Restore_Buffer()
* ------------------------------------------------------------------------------
* Overview: Function stores display content at current cursor location
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void HID_Mouse_Restore_Buffer(){
  if (HID_Mouse_Connected){
    Copy_DisplayUnderPointer(oldX, oldY);
  }
}

/*******************************************************************************
* Function MouseInit()
* ------------------------------------------------------------------------------
* Overview: Function initialize HID mouse
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void MouseInit(){

  if (USBHost_HIDGetDeviceType() == _USB_HOST_HID_DEVICE_MOUSE){
    // HID Mouse
    ;
  }
  else if (USBHost_HIDGetDeviceType() == _USB_HOST_HID_DEVICE_KEYB){
    // HID Keyboard
    ;
  }
}

/*******************************************************************************
* Function HID_MOUSE_UpdatePosition(int16_t x, int16_t y, int16_t b)
* ------------------------------------------------------------------------------
* Overview: Function updates position from data received from HID report
* Input: coordinates and button position
* Output: Nothing
*******************************************************************************/
void HID_MOUSE_UpdatePosition (int16_t x, int16_t y, int16_t b){

  cButton = b;
  if((x != 0) || (y!=0)){

    cX += x;
    cY += y;

    if((cX - pX)>_DELTA)
      cX = pX+_DELTA;
    if((cY - pY)>_DELTA)
      cY = pY+_DELTA;

    if((pX - cX)>_DELTA)
      cX = pX-_DELTA;
    if((pY - cY)>_DELTA)
      cY = pY-_DELTA;

    if(cX > _X_MAX - _MOUSE_SIZE) {
      cX = _X_MAX - _MOUSE_SIZE;
    }
    else if(cX < _X_MIN) {
      cX = _X_MIN;
    }
    if (cY > _Y_MAX - _MOUSE_SIZE) {
      cY = _Y_MAX - _MOUSE_SIZE;
    }
    if (cY < _Y_MIN) {
      cY = _Y_MIN;
    }

    pX = cX;
    pY = cY;
  }
}

/*******************************************************************************
* Function MouseReportReceived(uint8_t* reportData)
* ------------------------------------------------------------------------------
* Overview: Function parses HID report from HID mouse : HID report is specific
*           for each HID device. User can modify this function in order to work
*           with any HID Mouse.
* Input: pointer to report array
* Output: Nothing
*******************************************************************************/
void MouseReportReceived(uint8_t* reportData){
  mouseData.button = reportData[0] + ((uint16_t)reportData[1] << 8);
  mouseData.x = reportData[2] + ((uint16_t)reportData[3] << 8);
  mouseData.y = reportData[4] + ((uint16_t)reportData[5] << 8);
  
  HID_MOUSE_UpdatePosition(mouseData.x, mouseData.y, mouseData.button);
  HID_Mouse_flag = 1;
}

/*******************************************************************************
* Function HID_Mouse_Handler()
* ------------------------------------------------------------------------------
* Overview: Function handles mouse state
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void HID_Mouse_Handler() {
  if (HID_Mouse_Connected){
    if(HID_Mouse_flag) {
      HID_Mouse_flag = 0;

      // mouse right button is pressed
      if (cButton == 1){
        Xcoord = cX;
        Ycoord = cY;
        update = 1;
        // process press
        Process_TP_Press(cX, cY);
        if (My_PenDown == 0){
          update = 2;
          My_PenDown = 1;
          // process on down
          HID_Mouse_Restore_Display();
          Process_TP_Down(cX, cY);
        }
      }
      if (cButton == 0){
        if (My_PenDown == 1) {
          Xcoord = cX;
          Ycoord = cY;
          update = 2;
          My_PenDown = 0;
          // process on up
          HID_Mouse_Restore_Display();
          Process_TP_Up(cX, cY);
        }
        My_PenDown = 0;
      }
      // update mouse position
      if ((oldx != cx) || (oldy != cY) || (update > 0)){
        if (update != 2)
          Copy_Buffer_ToDisplay(oldx, oldy);
        if (update == 2)
          Copy_DisplayUnderPointer(cX, cY);
        Copy_DisplayUnderPointer(cX, cY);
        Copy_PointerToDisplay(cX, cY);
        oldx = cX;
        oldy = cY;
        update = 0;
      }
    }
  }
}

/*******************************************************************************
* Function HIDEventHandler(uint8_t event)
* ------------------------------------------------------------------------------
* Overview: Function handles mouse state
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void HIDEventHandler(uint8_t event){
  // HID device disconnected
  if(event == _USB_HOST_EVENT_DEVICE_DISCONNECTED){
    HID_Mouse_Connected = 0;
    Copy_Buffer_ToDisplay(cX, cY);
  }
  // HID device connected
  else if(event == _USB_HOST_EVENT_DEVICE_CONNECTED){
    asm nop;
  }
  // HID device enumerated
  else if(event == _USB_HOST_EVENT_DEVICE_ENUMERATED){
    HID_Mouse_Connected = 1;
    Copy_DisplayUnderPointer(cX, cY);
    Copy_PointerToDisplay(cX, cY);
  }
}

/*******************************************************************************
* Function HID_Mouse_Start(uint8_t event)
* ------------------------------------------------------------------------------
* Overview: Function starts HID Mouse
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void HID_Mouse_Start(){
unsigned int *ptr1, *ptr2;
unsigned int i;

  ptr1 = (unsigned int *)pointer_bmp;
  ptr1 += 3;
  ptr2 = pointer;
  for (i = 0; i < 400; i ++)
    *ptr2++ = *ptr1++;

  // if there is a voltage detected on the USB comm port, do not enable USB Host (USB host will provide voltage for the connected USB device)
  if (USB_VBUS == 1)
    return;

  GPIO_Config(&GPIO_PORTB_AHB, _GPIO_PINMASK_0 | _GPIO_PINMASK_1, _GPIO_DIR_INPUT, _GPIO_CFG_DIGITAL_ENABLE | _GPIO_CFG_PULL_DOWN | _GPIO_CFG_DRIVE_2mA, 0);
  GPIO_Digital_Output(&GPIO_PORTD_AHB, _GPIO_PINMASK_6);
  GPIO_Config(&GPIO_PORTL, _GPIO_PINMASK_6 | _GPIO_PINMASK_7, _GPIO_DIR_INPUT, _GPIO_CFG_ADV_AIN | _GPIO_CFG_DRIVE_2mA, 0);
  GPIO_PORTL_AMSEL = 0xC0;

  PSW = 1;
  Delay_ms(200);
  USBHost_HIDInit();
  USBHost_RegisterDeviceEventHandler(HIDEventHandler);

  PSW = 0;
  USBHost_Init();

  NVIC_IntEnable(IVT_INT_USB);
  EnableInterrupts();
}

/*******************************************************************************
* End of File
*******************************************************************************/