#include "mikromedia_5_TIVA_objects.h"
#include "Power_management.h"
//******************************************************************************
// hardware definitions
//******************************************************************************

// RGB Leds
sbit LED_R                     at GPIO_PORTD_AHB_DATA.B2;
sbit LED_G                     at GPIO_PORTD_AHB_DATA.B3;
sbit LED_B                     at GPIO_PORTD_AHB_DATA.B5;
sbit LED_R_Direction           at GPIO_PORTD_AHB_DIR.B2;
sbit LED_G_Direction           at GPIO_PORTD_AHB_DIR.B3;
sbit LED_B_Direction           at GPIO_PORTD_AHB_DIR.B5;

// MMC Connections
sbit Mmc_Chip_Select           at GPIO_PORTA_AHB_DATA.B3;
sbit MMC_Card_Detect           at GPIO_PORTE_AHB_DATA.B6;
sbit Mmc_Chip_Select_Direction at GPIO_PORTA_AHB_DIR.B3;
sbit MMC_Card_Detect_Direction at GPIO_PORTE_AHB_DIR.B6;

// MP3 Codec Connections
sbit MP3_CS                    at GPIO_PORTN_DATA.B0;
sbit MP3_RST                   at GPIO_PORTN_DATA.B2;
sbit BSYNC                     at GPIO_PORTN_DATA.B3;
sbit DREQ                      at GPIO_PORTN_DATA.B1;
sbit MP3_CS_Direction          at GPIO_PORTN_DIR.B0;
sbit MP3_RST_Direction         at GPIO_PORTN_DIR.B2;
sbit BSYNC_Direction           at GPIO_PORTN_DIR.B3;
sbit DREQ_Direction            at GPIO_PORTN_DIR.B1;

// nRF24L01+ connections
sbit nRF_CS                    at GPIO_PORTF_AHB_DATA.B6;
sbit nRF_CE                    at GPIO_PORTG_AHB_DATA.B3;
sbit nRF_IRQ                   at GPIO_PORTG_AHB_DATA.B2;
sbit nRF_CS_Direction          at GPIO_PORTF_AHB_DIR.B6;
sbit nRF_CE_Direction          at GPIO_PORTG_AHB_DIR.B3;
sbit nRF_IRQ_Direction         at GPIO_PORTG_AHB_DIR.B2;

// BackLight LED
sbit BackLight_LED             at GPIO_PORTM_DATA.B6;
sbit BackLight_LED_Direction   at GPIO_PORTM_DIR.B6;

// Power Management pin declarations
sbit USB_VBUS                  at GPIO_PORTB_AHB_DATA.B1;
sbit DC_VBUS                   at GPIO_PORTN_DATA.B4;
sbit BAT_STAT1                 at GPIO_PORTQ_DATA.B6;
sbit BAT_STAT2                 at GPIO_PORTQ_DATA.B7;
sbit USB_VBUS_Direction        at GPIO_PORTB_AHB_DIR.B1;
sbit DC_VBUS_Direction         at GPIO_PORTN_DIR.B4;
sbit BAT_STAT1_Direction       at GPIO_PORTQ_DIR.B6;
sbit BAT_STAT2_Direction       at GPIO_PORTQ_DIR.B7;

// WiFi connections
sfr sbit Net_Wireless_CC3000_CS  at GPIO_PORTB_AHB_DATA.B4;
sfr sbit Net_Wireless_CC3000_Int at GPIO_PORTB_AHB_DATA.B2;
sfr sbit Net_Wireless_CC3000_En  at GPIO_PORTF_AHB_DATA.B5;

sfr sbit Net_Wireless_CC3000_CS_Direction  at GPIO_PORTB_AHB_DIR.B4;
sfr sbit Net_Wireless_CC3000_Int_Direction at GPIO_PORTB_AHB_DIR.B2;
sfr sbit Net_Wireless_CC3000_En_Direction  at GPIO_PORTF_AHB_DIR.B5;

// USB Host
sbit PSW                       at GPIO_PORTD_AHB_DATA.B6;

// external resources
const unsigned long mikroProg_for_tiva_bmp       = 0x0000014C;
const unsigned long arm_compilers_bmp            = 0x00092F52;
const unsigned long clicks_bmp                   = 0x00125D58;
const unsigned long easymxprov7_bmp              = 0x001B8B5E;
const unsigned long mikromedia5_for_tiva_bmp     = 0x0024B964;
const unsigned long mikromediaplus_stellaris_bmp = 0x002DE76A;
const unsigned long mp3_note_bmp                 = 0x00371570;
const unsigned long mp3_player_note_bmp          = 0x003862B0;
const unsigned long game_table_bmp               = 0x00390546;
const unsigned long rgb_bitmap_bmp               = 0x003E2CEC;

// global flags
char Fat_Initialized_Flag;
unsigned long BackLight_PWM_period;
unsigned int  *My_MemoryStartPtr;

/******************************************************************************/

// static
static signed short Ext_fhandle;
static char Ext_res_initialized;
static unsigned long rfStartSect;
static char My_Ext_Data_Buffer[512];
static unsigned long currentSector = -1, res_file_size;

/*******************************************************************************
* Function Init_GPIO()
* ------------------------------------------------------------------------------
* Overview: Function Initialize GPIO ports
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void Init_GPIO(){
  // init MMC_Card_Detect
  GPIO_Digital_Input(&GPIO_PORTE_AHB, _GPIO_PINMASK_6);
  
  // init piezo
  Sound_Init(&GPIO_PORTM_DATA, 7);
  
  // init RGB
  GPIO_Digital_Output(&GPIO_PORTD_AHB, _GPIO_PINMASK_2 | _GPIO_PINMASK_3 | _GPIO_PINMASK_5);
  
  // Initiliaze Power management
  Power_Man_Init();
  
  // init Backlight Diode
  GPIO_Digital_Output(&GPIO_PORTM, _GPIO_PINMASK_6);
  BackLight_PWM_period = PWM_T5CCP0_Init(200, &_GPIO_MODULE_T5CCP0_M6);
  PWM_T5CCP0_Set_Duty(BackLight_PWM_period, _PWM_INVERTED_DISABLE);
  PWM_T5CCP0_Start();
  
  // Set default values for flags
  Fat_Initialized_Flag = 0;
  Ext_res_initialized = 0;
  Ext_fhandle = 0xFF;
  
  My_MemoryStartPtr = 0x10000000;
}

/*******************************************************************************
* Function MyTFT_Get_Data(unsigned long offset, unsigned long count, unsigned long *num)
* ------------------------------------------------------------------------------
* Overview: Function Sets Get Data Handler for FAT32 format
* Input: offset in RES file, number of bytes to read, number of read bytes
* Output: Pointer to desired position in buffer
*******************************************************************************/
char* MyTFT_Get_Data(unsigned long fPos, unsigned long bytesToGet, unsigned long *pBytesGot) {
  unsigned int  scOffset;

  *pBytesGot = 0;

  if (Ext_fhandle < 0) {
    return 0; // return with nil if handle was bad.
  }

  SPI_Set_Active(SPI0_Read, SPI0_Write);
  // We will utilize some of the fat32 implied features
  // (such as built in sector buffer) to reduce ram memory consumpiton.
  // You can't use this approach if this buffer is going to be used
  // from another thread (i.e. using fat32 routines in interrupt...)
  // In that case or if you have plenty of ram memory use separate
  // as large as possible data buffer.
  f32_sector.fSectNum = rfStartSect + fPos / 512;
  Mmc_Read_Sector(f32_sector.fSectNum, f32_sector.fSect);

  // 512 bytes sector buffer implied.
  scOffset = fPos % 512;
  if(bytesToGet > 512-scOffset)
    *pBytesGot = 512-scOffset;
  else
    *pBytesGot = bytesToGet;

  SPI_Set_Active(SPI1_Read, SPI1_Write);

  return &f32_sector.fSect + scOffset;
}

/*******************************************************************************
* Function Fill_LoadingBar(unsigned short bar)
* ------------------------------------------------------------------------------
* Overview: Function draws Loading bar during coping of resources
* Input: bar position (0..100)
* Output: Nothing
*******************************************************************************/
void Fill_LoadingBar(unsigned short bar){
  static unsigned short old_bar;
  
  if (bar == 127){
    TFTIntern_Set_Pen(CL_GRAY_Intern, 2);
    TFTIntern_H_Line(Line1.First_Point_X - 5, Line1.Second_Point_X, Line1.Second_Point_Y);
  }
  else
    if (old_bar != bar){
      old_bar = bar;
      TFTIntern_Set_Pen(CL_WHITE_Intern, 2);
      TFTIntern_H_Line(Line1.First_Point_X + (bar-1) * 5, Line1.First_Point_X + bar * 5, Line1.Second_Point_Y);
    }
}

/*******************************************************************************
* Function CopyImage_FromResources_ToSRAM(unsigned long offset, unsigned int *SRAMStartPtr)
* ------------------------------------------------------------------------------
* Overview: Function Copies resource from SD card into SRAM at desired location
* Input: offset of desired resource in RES file, address in SRAM as final location
* Output: Nothing
*******************************************************************************/
char CopyImage_FromResources_ToSRAM(unsigned long offset, unsigned int *SRAMStartPtr){
  unsigned long number, written;
  char *ptr;
  signed long size;
  unsigned int step;

  ptr = MyTFT_Get_Data(offset, 6, &number);
  // calculate Image size
  if (number == 6){
    size = ((unsigned long)(*(ptr+3)) << 8) + *(ptr+2);
    size *= ((unsigned long)(*(ptr+5)) << 8) + *(ptr+4);
    size *= 2;
    size += 6;                                   // header is 6 bytes
  }
  else 
    return 1;
  
  step = size / 100;
  written = 0;
  while (size > 0){
    if (size%2)
      asm nop;
    ptr = MyTFT_Get_Data(offset + written, 512, &number);
    EPI_WorkaroundHWordWriteArrayInc(SRAMStartPtr + written/2, (unsigned int *)ptr, number/2);
    Fill_LoadingBar(written / step);
    size -= number;
    written += number;
  }
  
  Fill_LoadingBar(127);
  return 0;
}

/*******************************************************************************
* Function Get_Cursor_Offset(unsigned int _x, unsigned int _y)()
* ------------------------------------------------------------------------------
* Overview: Function returns offset of the current pixel in TFT memory buffer
* Input: coordinates of current pixel (cursor)
* Output: offset in TFT memory buffer
*******************************************************************************/
unsigned long Get_Cursor_Offset(unsigned int _x, unsigned int _y) {
unsigned int x_cord, y_cord;

  if (TFTIntern_GetDisplayRotation() == _TFT_INTERN_ROTATE_0) {
    x_cord = _x;
    y_cord = _y;
  } else if (TFTIntern_GetDisplayRotation() == _TFT_INTERN_ROTATE_180) {
    x_cord = TFTIntern_DisplayWidth - 1 - _x;
    y_cord = TFTIntern_DisplayHeight - 1 - _y;
  }
  
  return (TFTIntern_DisplayWidth * (unsigned long)y_cord) + x_cord;
}

/*******************************************************************************
* Function Init_FAT()
* ------------------------------------------------------------------------------
* Overview: Function Initialize FAT32 system
* Input: Nothing
* Output: 1 if external resources file successfully opened
*         0 if not successful
*******************************************************************************/
char Init_FAT(){
char FAT_cnt = 0;
  if (MMC_Card_Detect == 0){
    if (Fat_Initialized_Flag == 0){
      SPI0_Init_Advanced(1000000, _SPI_MASTER, _SPI_8_BIT | _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION, &_GPIO_MODULE_SPI0_A245_AHB);
      while ((Fat32_Init() != 0) && (FAT_cnt < 5))
        Fat_cnt ++;
      if (FAT_cnt < 5){
        SPI0_Init_Advanced(20000000, _SPI_MASTER, _SPI_8_BIT | _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION, &_GPIO_MODULE_SPI0_A245_AHB);
        Fat_Initialized_Flag = 1;
      }
    }
    if ((Ext_res_initialized == 0) && (Fat_Initialized_Flag == 1)){
      TFTIntern_Set_Ext_Buffer(MyTFT_Get_Data);
      Ext_fhandle = FAT32_Open("Res_reso.RES", 1);
      rfStartSect = FAT32_ClustToSect(fat32_fdesc[Ext_fhandle]._1stClust);
      if (Ext_fhandle != 0xFF)
        Ext_res_initialized = 1;
    }
  }
  SPI0_Init_Advanced(20000000, _SPI_MASTER, _SPI_8_BIT | _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION, &_GPIO_MODULE_SPI0_A245_AHB);
  SPI_Set_Active(SPI1_Read, SPI1_Write);
  return Ext_res_initialized;
}

/*******************************************************************************
* Function RedrawLabel(unsigned int BackGround_Color, Tlabel *Label, unsigned int Xoffset, unsigned int Yoffset)
* ------------------------------------------------------------------------------
* Overview: Function Initializes Sensors example
* Input: Beackground color, pointer to Label,offset which should be redrawn
* Output: Nothing
*******************************************************************************/
void RedrawLabel(unsigned int BackGround_Color, Tlabel *Label, unsigned int Xoffset, unsigned int Yoffset){
  TFTIntern_Set_Brush(1, BackGround_Color, 0, 0, 0, 0);
  TFTIntern_Set_Pen(BackGround_Color, 0);
  TFTIntern_Rectangle(label->Left, Label->Top, label->Left + Xoffset, Label->Top + Yoffset);
  DrawLabel(Label);
}

/*******************************************************************************
* Function: BLED_Fade_Out
* ------------------------------------------------------------------------------
* Overview: Function performs fade out on display backlight
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void BLED_Fade_Out(){
  signed short i;

  i = 127;
  while (i > 0){
    PWM_T5CCP0_Set_Duty((BackLight_PWM_period / 128) * i, _PWM_INVERTED_DISABLE);
    i -= 2;
    Delay_ms(1);
  }
  PWM_T5CCP0_Set_Duty(1, _PWM_INVERTED_DISABLE);
}

/*******************************************************************************
* Function: BLED_Fade_In
* ------------------------------------------------------------------------------
* Overview: Function performs fade in on display backlight
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void BLED_Fade_In(){
  signed short i;

  i = 1;
  while (i < 124){
    PWM_T5CCP0_Set_Duty((BackLight_PWM_period / 128) * i, _PWM_INVERTED_DISABLE);
    i += 2;
    Delay_ms(1);
  }
  PWM_T5CCP0_Set_Duty(BackLight_PWM_period, _PWM_INVERTED_DISABLE);
}

/*******************************************************************************
* End of File
*******************************************************************************/