/*******************************************************************************
* MP3_main  - Function Set used for MP3 player
* ------------------------------------------------------------------------------
*
*******************************************************************************/
#include "mikromedia_5_TIVA_objects.h"
#include "mikromedia_5_TIVA_resources.h"
#include "Init_routines.h"
#include "MP3_Driver.h"
#include "HID_Mouse.h"
#include "__Lib_FAT32.h"


// globals
char MP3_Example_State;
char level = 100, old_level = 0;
signed short current_file;
char MP3_Image_Loaded = 0;
unsigned int *MP3_Image1_SRAM_ptr = 0x105D0000;
unsigned int *MP3_Image2_SRAM_ptr = 0x105C0000;

typedef struct Scroll_List {
  char FileName[13];
  char FileType;
} TScroll_List;
extern TScroll_List File_List[];
extern char Total_Entries;

// extern functions
char Get_Next_File_Name(signed short number, char *Text);
void UpdateVolumeBar(char vol, char old_vol);
void Initialize_Scroll();

// static
static const BYTES_2_WRITE = 32;
static const BUFFER_SIZE = 256;

TImage *Images[3] = {&Image12, &Image13, &Image14};
static unsigned long Buffer_count, Total_Buffers;
static char MP3_song[13];
static char current_fhandle;
static int MP3_Example_cnt = 0;
static unsigned long file_size;
static char mp3_buffer[BUFFER_SIZE];
static char volume_left, volume_right;
static unsigned int mp3_secs, mp3_old_secs;
static char old_Progress = 0;
static char MP3_counter = 0;

void Mp3_Init();
void MP3_Start();
static void MP3_Check_Time();
void MP3_Reset_Time();

/*******************************************************************************
* Function MP3_play(char *Song_Name, char File_No)
* ------------------------------------------------------------------------------
* Overview: Function starts to play new mp3 file
* Input: File name, number of file in the list
* Output: Nothing
*******************************************************************************/
void MP3_play(char *Song_Name, char File_No){
  SPI_Set_Active(SPI0_Read, SPI0_Write);

  current_file = File_No;
  Buffer_count = 0;
  current_fhandle = FAT32_Open( Song_Name, FILE_READ );
  FAT32_Size(Song_Name, &file_size);
  Buffer_count = 0;
  Total_Buffers = File_size / BUFFER_SIZE;
  mp3_secs = 0;
  MP3_Reset_Time();
  MP3_Check_Time();
  MP3_Set_Volume(100 - level, 100 - level);
  UpdateVolumeBar( level, 0);
  MP3_Example_State = 2;
  
  SPI_Set_Active(SPI1_Read, SPI1_Write);
}

/*******************************************************************************
* Function Start_MP3_Player(char *Song_Name, char File_No)
* ------------------------------------------------------------------------------
* Overview: Function starts to play new mp3 file
* Input: File name, number of file in the list
* Output: Nothing
*******************************************************************************/
void Start_MP3_Player(char *Song_Name, char File_No){
  strcpy(Label28_Caption, Song_Name);
  DrawScreen(&MP3_Player);

  TFTIntern_Image(330, 115, MP3_Image2_SRAM_ptr, 1);

  pause_icon.Active = 1;
  Play_icon.Active = 0;
  DrawImage(&pause_icon);
  MP3_play(Song_Name, File_No);
}

/*******************************************************************************
* Function StopMP3_File()
* ------------------------------------------------------------------------------
* Overview: Function stops playing and closes mp3 file
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void StopMP3_File(){
  SPI_Set_Active(SPI0_Read, SPI0_Write);

  mp3_secs = 0;
  MP3_Reset_Time();
  FAT32_Close(current_fhandle);
  MP3_Example_State = 3;
  
  SPI_Set_Active(SPI1_Read, SPI1_Write);
}

/*******************************************************************************
* Function GetNext_mp3_File(char Next_Previous)
* ------------------------------------------------------------------------------
* Overview: Function gets next audio file for playing
* Input: flag if we should get next or previous file from list
* Output: Nothing
*******************************************************************************/
void GetNext_mp3_File(char Next_Previous){      // 0 for previous, 1 for next
char temp = 0, temp1;
  StopMP3_File();
  temp1 = current_file;
  if (Next_Previous)
    while (temp == 0 && temp != 2)
      temp = Get_Next_File_Name( ++current_file, MP3_song);
  else
    while (temp == 0 && temp != 2)
      temp = Get_Next_File_Name( --current_file, MP3_song);
  if (temp == 1){
    MP3_play(MP3_song, current_file);
    strcpy(Label28_Caption, MP3_song);
    RedrawLabel(MP3_Player.Color, &Label28, 200, 40);
  }
  else{
    if (Next_Previous)
      current_file = 0xFF;
    else
      current_file = Total_Entries + 1;
    MP3_Song[0] = 0;
    strcpy(Label28_Caption, MP3_song);
    RedrawLabel(MP3_Player.Color, &Label28, 200, 40);
  }
}

/*******************************************************************************
* Function MP3_Check_Time()
* ------------------------------------------------------------------------------
* Overview: Function updates progress bar on Player screen
* Input: Nothing
* Output: Nothing
*******************************************************************************/
static void MP3_Check_Time(){

  MP3_Get_Decode_Time(&mp3_secs);

  if ((mp3_secs > mp3_old_secs) && (mp3_secs < mp3_old_secs + 2)){
    Label6_Caption[0] = (mp3_secs / 60) / 10 + 48;
    Label6_Caption[1] = (mp3_secs / 60) % 10 + 48;
    Label6_Caption[3] = (mp3_secs % 60) / 10 + 48;
    Label6_Caption[4] = (mp3_secs % 60) % 10 + 48;
    RedrawLabel(MP3_player.Color, &Label6, 70, 30);
    mp3_old_secs = mp3_secs;
  }
}

/*******************************************************************************
* Function UpdateProgressBar()
* ------------------------------------------------------------------------------
* Overview: Function updates progress bar on Player screen
* Input: Nothing
* Output: Nothing
*******************************************************************************/
static void UpdateProgressBar(){
unsigned int progress;
  progress = (Buffer_count * 400) / Total_Buffers;
  if (progress != old_Progress){
    if ((progress > 0) && (progress < 401)){
      TFTIntern_Set_Brush(1, 0x55FF, 0, 0, 0, 0);
      TFTIntern_Set_Pen(0x55FF, 0);
      TFTIntern_Rectangle(ProgressBar.Left, ProgressBar.Top, ProgressBar.Left + progress, ProgressBar.Top + ProgressBar.Height);

      TFTIntern_Set_Brush(1, ProgressBar.Color, 0, 0, 0, 0);
      TFTIntern_Set_Pen(ProgressBar.Color, 0);
      TFTIntern_Rectangle(ProgressBar.Left + progress + 1, ProgressBar.Top, ProgressBar.Left + ProgressBar.Width, ProgressBar.Top + ProgressBar.Height);

      old_progress = progress;
    }
  }
}

/*******************************************************************************
* Function doMP3()
* ------------------------------------------------------------------------------
* Overview: Function runs mp3 example. Should be called frequently
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void doMP3(){
 unsigned long i;
 
  if (MP3_counter++ > 100){
    switch (MP3_Example_State){
      case 0 : { // mp3 example started
                 if (MMC_Card_Detect == 0){        // if SD card detected
                   HID_Mouse_Restore_Display();
                   DrawScreen(&MP3_SongList);
                   TFTIntern_Image(450, 130, MP3_Image1_SRAM_ptr, 1);
                   HID_Mouse_Restore_Buffer();
                   Delay_ms(100);
                   Mp3_Init();
                   MP3_Example_State = 1;
                 }
                 else{                             // if there is no SD card
                   DrawImage(Images[MP3_Example_cnt / 200]);
                   MP3_Example_cnt++;
                   if (MP3_Example_cnt > 599)
                     MP3_Example_cnt = 0;
                 }
               }; break;
      case 1 : { // stop playing
               }; break;                           // if mp3 is stopped, do nothing
      case 2 : { // play song
                 SPI_Set_Active(SPI0_Read, SPI0_Write);
                 if (file_size > BUFFER_SIZE){     // while there is still data, send it to mp3 codec chip
                   Fat32_Read(current_fhandle, mp3_buffer, BUFFER_SIZE);
                   for (i=0; i<BUFFER_SIZE/BYTES_2_WRITE; i++) {
                     // Initialize SPI0 module at lower speed
                     MP3_SDI_Write_32(mp3_buffer + i*BYTES_2_WRITE);
                   }
                   file_size -= BUFFER_SIZE;
                   Buffer_count ++;
                 }
                 UpdateProgressBar();              // update progress bar
                 MP3_Check_Time();                 // update elapsed time
                 if (file_size < BUFFER_SIZE){     // finish playing file
                   Fat32_Read(current_fhandle, mp3_buffer, file_size);
                   for (i=0; i<file_size; i++){
                     // Initialize SPI0 module at lower speed
                     MP3_SDI_Write(mp3_buffer[i]);
                   }
                   MP3_Example_State = 1;
                   Buffer_count ++;
                   UpdateProgressBar();
                 }
                 // Initialize SPI2 module at higher speed
                 SPI_Set_Active(SPI1_Read, SPI1_Write);

               }; break;
      case 3 : { // pause playing
               }; break;

    }
    MP3_counter = 0;
  }
}

/*******************************************************************************
* Function Mp3_Init()
* ------------------------------------------------------------------------------
* Overview: Function Starts MP3 example (draws appropriate Screen and sets state)
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void Mp3_Init(){

  Init_FAT();
  if (MMC_Card_Detect == 1){
    DrawScreen(&MP3_SD);
    MP3_Example_State = 0;
    return;
  }
  MP3_Example_State = 1;
  Initialize_Scroll();
  MP3_Start();
}

/*******************************************************************************
* Function MP3_Start()
* ------------------------------------------------------------------------------
* Overview: Function Initialize SPI and MP3 codec chip for operation
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void MP3_Start() {

  // MP3_CS, MP3_RST and BSYNC are outputs
  GPIO_Digital_Output(&GPIO_PORTN, _GPIO_PINMASK_0 | _GPIO_PINMASK_2 | _GPIO_PINMASK_3);
  // DREQ is input
  GPIO_Digital_Input(&GPIO_PORTN, _GPIO_PINMASK_1);
  
  MP3_CS            = 1;               // Deselect MP3_CS
  MP3_RST           = 1;               // Set MP3_RST pin
  BSYNC             = 0;               // Clear BSYNC
  BSYNC             = 1;               // Clear BSYNC

  // Initialize SPI0 module at lower speed
  SPI0_Init_Advanced(2000000, _SPI_MASTER, _SPI_8_BIT | _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION, &_GPIO_MODULE_SPI0_A245_AHB);

  MP3_Set_default_Mode();

  // Initialize SPI2 module at higher speed
  SPI0_Init_Advanced(10000000, _SPI_MASTER, _SPI_8_BIT | _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION, &_GPIO_MODULE_SPI0_A245_AHB);
  
  SPI_Set_Active(SPI1_Read, SPI1_Write);
                     
  MP3_Set_Volume(100 - level, 100 - level);
  mp3_secs = 0;
  MP3_Reset_Time();
}

/*******************************************************************************
* End of File
*******************************************************************************/