/*******************************************************************************
* Power Management  - Function Set used for Power Management
* ------------------------------------------------------------------------------
* reads battery voltage gets power source
*******************************************************************************/
#include "mikromedia_5_TIVA_objects.h"
#include "Init_routines.h"

// globals
char Display_Bat_Stat = 1;

// static
// TypeDefs
typedef enum { BAT_CHARGING, BAT_DISCHARGING, BAT_UNPLUGGED, BAT_FULL, BAT_BADBATTERY } Batt_Status;
typedef enum { SRC_BATTERY, SRC_USB, SRC_DC } Pwr_Source;

struct    { Batt_Status   Battery_state;
            Pwr_Source    Source;
            unsigned long ui_Voltage;
          } Power_State, Power_State_Old;



/*******************************************************************************
* Function Power_Man_Init()
* ------------------------------------------------------------------------------
* Overview: Function initializes input pins for Power Management
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void Power_Man_Init() {
  GPIO_Analog_Input(&GPIO_PORTK, _GPIO_PINMASK_3);
  ADC_Set_Input_Channel(_ADC_CHANNEL_19);
  ADC1_Init_Advanced(_ADC_EXTERNAL_REF);                       // Initialize ADC

  GPIO_Config(&GPIO_PORTB_AHB, _GPIO_PINMASK_1, _GPIO_DIR_INPUT, _GPIO_CFG_DIGITAL_ENABLE | _GPIO_CFG_PULL_DOWN | _GPIO_CFG_DRIVE_2mA, 0);
//  GPIO_Digital_Input(&GPIO_PORTB_AHB, _GPIO_PINMASK_1);        // USB_VBUS
  GPIO_Digital_Input(&GPIO_PORTN, _GPIO_PINMASK_4);            // DC_VBUS
  GPIO_Digital_Input(&GPIO_PORTQ, _GPIO_PINMASK_6 | _GPIO_PINMASK_7);            // BAT_STAT1 & BAT_STAT2

  Power_State_Old.Battery_state = -1;
  Power_State_Old.Source = -1;
  Power_State_Old.ui_Voltage = -1;
}

/*******************************************************************************
* Function ui_ReadBatteryVoltage()
* ------------------------------------------------------------------------------
* Overview: Function reads Battery Voltage and returns integer value
* Input: Nothing
* Output: Battery Voltage
*******************************************************************************/
static unsigned int ui_ReadBatteryVoltage() {
  unsigned int ui_voltage;
  char cntr;

  for(cntr=0, ui_voltage=0; cntr<8; cntr++) {
    ui_voltage += ADC1_Get_Sample(19);
    Delay_10ms();
  }
  return ui_voltage >> 3;
}

/*******************************************************************************
* Function f_ReadBatteryVoltage()
* ------------------------------------------------------------------------------
* Overview: Function reads Battery Voltage and returns float value
* Input: Nothing
* Output: Battery Voltage
*******************************************************************************/
static unsigned long f_ReadBatteryVoltage() {
  int ui_voltage;

  ui_voltage = ui_ReadBatteryVoltage();
  return (unsigned long)ui_voltage*15;         // 3*2.048/4096 * 10000
}

/*******************************************************************************
* Function GetPowerStatus()
* ------------------------------------------------------------------------------
* Overview: Function sets power status
* Input: Nothing
* Output: Nothing
*******************************************************************************/
static void GetPowerStatus() {

  Power_State.ui_Voltage  = f_ReadBatteryVoltage();

  if (DC_VBUS==0)
    Power_State.Source = SRC_DC;      // Works if Vdc>5V(Vusb), if Vdc<Vusb priority is changed
  else {
    if (USB_VBUS==1)
      Power_State.Source = SRC_USB;
    else
      Power_State.Source = SRC_BATTERY;
  }

  if (BAT_STAT1 == 1)
  {
    if (BAT_STAT2 == 1)
      if (Power_State.Source == SRC_BATTERY)
        Power_State.Battery_state = BAT_DISCHARGING;
      else
        Power_State.Battery_state = BAT_UNPLUGGED;
    else
      Power_State.Battery_state = BAT_FULL;
  }
  else
  {
    if (BAT_STAT2 == 1)
      Power_State.Battery_state = BAT_CHARGING;
    else
      Power_State.Battery_state = BAT_UNPLUGGED; //BAT_BADBATTERY;
  }
}

/*******************************************************************************
* Function checkPowerM()
* ------------------------------------------------------------------------------
* Overview: Function update battery status
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void checkPowerM(){
  int Bar_Width;

  GetPowerStatus();
  
  if (Power_State.Battery_state != Power_State_Old.Battery_state) {
    switch (Power_State.Battery_state) {
      case BAT_CHARGING   : {
//                              strcpy(Label44_Caption, "Charging");
//                              RedrawLabel(Home.Color, &Label44, 60, 13);
                            }; break;
      case BAT_DISCHARGING: {
//                              strcpy(Label44_Caption, "Discharging");
//                              RedrawLabel(Home.Color, &Label44, 60, 13);
                            }; break;
      case BAT_FULL       : {
//                              strcpy(Label44_Caption, "Bat Full");
//                              RedrawLabel(Home.Color, &Label44, 60, 13);
                            }; break;
      case BAT_UNPLUGGED  : {
//                              strcpy(Label44_Caption, "No Bat");
//                              RedrawLabel(Home.Color, &Label44, 60, 13);
                            }; break;
      case BAT_BADBATTERY  : {
//                              strcpy(Label44_Caption, "Bad Bat");
//                              RedrawLabel(Home.Color, &Label44, 60, 13);
                            }; break;
    }
    Power_State_Old.Battery_state = Power_State.Battery_state;
  }

  if (Power_State.Source != Power_State_Old.Source) {
    switch (Power_State.Source) {
      case SRC_BATTERY: {
//                              strcpy(Label45_Caption, "BAT");
//                              RedrawLabel(Home.Color, &Label45, 60, 13);
                        }; break;
      case SRC_DC     : {
//                              strcpy(Label45_Caption, "DC");
//                              RedrawLabel(Home.Color, &Label45, 60, 13);
                        }; break;
      case SRC_USB    : {
//                              strcpy(Label45_Caption, "USB");
//                              RedrawLabel(Home.Color, &Label45, 60, 13);
                        }; break;
    }
    Power_State_Old.Source = Power_State.Source;
  }

  // maximum will be for VBAT = 4.15V, and  minimum for VBAT 2.95V (red is for VBAT < 3.3V)
//  if (abs(Power_State.ui_Voltage - Power_State_Old.ui_Voltage) > 1000){
    Bar_Width = ((signed int)Power_State.ui_Voltage - 29000) / 300;
    if (Bar_Width > 40)
      Bar_Width = 40;
    if (Bar_width < 1)
      Bar_Width = 1;

    if (Power_State.ui_Voltage > 330)
      TFTIntern_Set_Brush(1, CL_WHITE_Intern, 0, 0, 0, 0);
    else
      TFTIntern_Set_Brush(1, CL_RED_Intern, 0, 0, 0, 0);

    TFTIntern_Set_Pen(CL_WHITE_Intern, 0);
    TFTIntern_Rectangle(33, 40, 33 + Bar_width, 51);

    TFTIntern_Set_Brush(1, Home.Color, 0, 0, 0, 0);
    TFTIntern_Set_Pen(CL_WHITE_Intern, 0);
    TFTIntern_Rectangle(32 + Bar_width, 40, 72, 51);
    
    Power_State_Old.ui_Voltage = Power_State.ui_Voltage;
}

/*******************************************************************************
* End of File
*******************************************************************************/