/*******************************************************************************
* RTC Driver - Function Set used for RTC module
* ------------------------------------------------------------------------------
*
*******************************************************************************/
#include "__Time.h"
// Time struct with am/pm
typedef struct Time {
  short ampm;
  short seconds;
  short minutes;
  short hours;
  short day;
  short month;
  int year;
} TTime;

TTime MyTime;

// static
static unsigned long RTC_Time, old_RTC_Time;
static unsigned long RTC_Date, old_RTC_Date;

/*******************************************************************************
* Function HibernateRTCSet()
* ------------------------------------------------------------------------------
* Overview: Function Sets desired Time into hibernate registers
* Input: time
* Output: Nothing
*******************************************************************************/
static void HibernateRTCSet(TimeStruct *temp) {
    unsigned long tmptmp = 0;

    tmptmp = (unsigned long)temp->hh*3600;
    tmptmp += temp->mn*60;
    tmptmp += temp->ss;

    while(!HIB_CTL_WRC_bit)
      ;
    HIB_RTCLD = tmptmp;
}

/*******************************************************************************
* Function HibernateRTCEnable()
* ------------------------------------------------------------------------------
* Overview: Function Enables Hibernate module
* Input: Nothing
* Output: Nothing
*******************************************************************************/
static void HibernateRTCEnable() {
    while(!HIB_CTL_WRC_bit);
    HIB_CTL = 0x40;                                  // Enable 32.768kHz osc
    while(!HIB_CTL_WRC_bit);
    HIB_RTCM0 = 86400;                               // 24 hour equivalent
    while(!HIB_CTL_WRC_bit);
    HIB_IM = 0x01;                                   // Set interrupt mask
    while(!HIB_CTL_WRC_bit);
    HIB_CTL = 0x41;                                  // Turn on counter
}

/*******************************************************************************
* Function HibernateRTCGet()
* ------------------------------------------------------------------------------
* Overview: Function reads Time from register
* Input: pointer ti Time struct
* Output: Nothing
*******************************************************************************/
static unsigned long HibernateRTCGet(TimeStruct *temp) {
   unsigned long clk;

   clk = HIB_RTCC;
   temp->hh   = (clk%86400)/3600;
   temp->mn = (clk%3600)/60;
   temp->ss = clk%60;

   return clk;
}

/*******************************************************************************
* Function RTC_Init()
* ------------------------------------------------------------------------------
* Overview: Function initializes RTC module
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void RTC_Init(){
TimeStruct My_Time;

  My_Time.ss = 15;
  My_Time.mn = 35;
  My_Time.hh = 11;
  My_Time.md = 10;
  My_Time.mo = 4;
  My_Time.yy = 2013;
  if ((HIB_CTL & 0x40) != 0x40){
    HibernateRTCSet(&My_Time);
  }
  HibernateRTCEnable();
}

/*******************************************************************************
* Function Set_RTC(TTime *RTCC_Time)
* ------------------------------------------------------------------------------
* Overview: Function Sets RTC Time and calendar
* Input: Time struct
* Output: Nothing
*******************************************************************************/
void Set_RTC(TTime *RTCC_Time){
TimeStruct My_Time;

  My_Time.ss = RTCC_Time->seconds;
  My_Time.mn = RTCC_Time->minutes;
  My_Time.hh = RTCC_Time->hours + RTCC_Time->ampm * 12;
  My_Time.md = RTCC_Time->day;
  My_Time.mo = RTCC_Time->month;
  My_Time.yy = RTCC_Time->year;

  HibernateRTCSet(&My_Time);
}

/*******************************************************************************
* Function RTCC_Read(TTime *RTCC_Time)
* ------------------------------------------------------------------------------
* Overview: Function reads RTC Time and calendar
* Input: Time struct
* Output: Nothing
*******************************************************************************/
char RTCC_Read(TTime *RTCC_Time){
  char temp = 0;
  TimeStruct My_Time;

  RTC_Time = HibernateRTCGet(&My_Time);

  if (RTC_Time != Old_RTC_Time){
    RTCC_Time->seconds = my_time.ss;
    RTCC_Time->minutes = my_time.mn;
    if (my_time.hh == 0){
      RTCC_Time->ampm  = 0;
      RTCC_Time->hours = 12;
    }
    else if (my_time.hh == 12){
      RTCC_Time->ampm  = 1;
      RTCC_Time->hours = 12;
    }
    else{
      RTCC_Time->hours   = my_time.hh % 12;
      RTCC_Time->ampm    = my_time.hh / 12;
    }
    RTCC_Time->day     = my_time.md;
    RTCC_Time->month   = my_time.mo;
    RTCC_Time->year    = my_time.yy;

    Old_RTC_Time = RTC_Time;
    temp = 1;
  }

  return temp;
}

/*******************************************************************************
* End of File
*******************************************************************************/