/*******************************************************************************
* RTC  - Function Set for RTC Example
* ------------------------------------------------------------------------------
* Example allows user to set up Time (RTCC module)
*   Also it allows users to connect to Time server over ethernet and synronize
*   with GMT Time.
*******************************************************************************/
#include "mikromedia_5_TIVA_objects.h"
#include "mikromedia_5_TIVA_resources.h"
#include "Init_routines.h"
#include "RTCC_driver.h"
#include "HID_Mouse.h"
#include "WiFi_main.h"

//externals
extern unsigned int Xcoord, Ycoord;

// types
typedef struct  RollingWheel {
  unsigned int  Left;
  unsigned int  Top;
  unsigned int  Width;
  unsigned int  Height;
  unsigned int  ImageHeight;
  const code char    *Picture_Name;
  unsigned int  currentlevel;
  int          active;
  int           position;
  char          levels;
} TRoller;

// rollers
TRoller Hour1Roll, Hour2Roll, Minute1Roll, Minute2Roll, AmPm;

static void Roll_Roller(TRoller *Roller, char dir, char step);
void Draw_Picture(TRoller *Image, int line, int xcoo, int ycoo);
void Roll_Get_Level(TRoller *Roller);
void Ethernet_Message(char *text);
static void check_time_server();

// static
char Roll_flag;
unsigned int OldYcoo, OldDelta;
TTime OldMyTime = {0, 0, 0, 0, 0, 0};

// const tables for displaying roller (should be long as half of roller height)
static const unsigned int table[] = { 13,13,12,12,11,11,10,10,10, 9, 9, 9, 8, 8, 8, 8, 7, 7, 7, 7, 7, 6, 6, 6, 6, 6, 6, 5, 5, 5, 5, 5, 5, 5, 5, 4, 4,
                                       4, 4, 4, 4, 4, 4, 4, 4, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1, 1, 1,
                                       1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                                       0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                                       0, 0, 0, 0, 0, 0, 0, 0,
                                       };

/*******************************************************************************
* Function MyBoxOnPress(TRoller *Roller)
* ------------------------------------------------------------------------------
* Overview: Function that scrolls desired roller for appropriate number of lines
* Input: Pointer to roller
* Output: Nothing
*******************************************************************************/
void MyBoxOnPress(TRoller *Roller){
int delta;
char dir, step;
  // if the roller was pressed for first Time save position and exit
  if (Roll_Flag == 0){
    Roll_Flag = 1;
    OldYcoo = Ycoord;
    OldDelta = 0;
    return;
  }
  // calculate difference in vertical axis and get direction
  delta = Ycoord - OldYcoo;
  if (delta > 0){
    dir = 0;
  }
  if (delta < 0){
    dir = 1;
    delta = - delta;
  }
  // if difference is too small exit
  if (delta < 4){
    OldDelta = delta;
    return;
  }
  // set rolling step
  step = 2;
  // if difference is large speed up scrolling
  if ((delta - OldDelta) > 5)
    delta += OldDelta + 5;
  else
    delta = delta + OldDelta;

  if (delta > 25){
    step = 3;
    if (delta > 40)
      step = 4;
    delta = 3 * delta;
  }
  delta = delta / 2;
  // roll for calculated number of lines
  while (delta > 0){
    Roll_Roller(Roller, dir, step);
    delta = delta - step;
  }
  // save latest vertical position
  OldYcoo = Ycoord;
  OldDelta = 0;
}

/*******************************************************************************
* Function DrawPictureLine(TRoller *Image, int line, int xcoo, int ycoo)
* ------------------------------------------------------------------------------
* Overview: Function draw desired picture line of a roller
* Input: pointer to roller, desired line of roller picture, coordinates where it should be drawn
* Output: Nothing
*******************************************************************************/
static void DrawPictureLine(TRoller *Image, int line, int xcoo, int ycoo){
  long count;
  const code char *ptr;
  char *ptr1;
  unsigned long offset;

  count = (Image->Width * 2);
  ptr1 = (char*)TFTIntern_rowData;
  ptr = Image->Picture_Name + 6 + (unsigned long)line * count;
  
  while (count --)
    *ptr1++ = *ptr++;
  
  offset = Get_Cursor_Offset(xcoo, ycoo);
  TFTIntern_Move_Cursor(xcoo, ycoo);
  EPI_WorkaroundHWordWriteArrayDec(My_MemoryStartPtr + offset, TFTIntern_rowData, Image->Width);
//  EPIWorkaroundHWordWriteArrayInc(My_MemoryStartPtr + offset, _rowData, Image->Width);
}

/*******************************************************************************
* Function DrawPictureLine(TRoller *Image, int line, int xcoo, int ycoo)
* ------------------------------------------------------------------------------
* Overview: Function draw entire roller from specified picture line
* Input: pointer to roller, desired line of roller picture, coordinates where it should be drawn
* Output: Nothing
*******************************************************************************/
static void Draw_Picture(TRoller *Image, int line, int xcoo, int ycoo){
signed int i;
int temp;

  for (i = 0; i < (Image->Height)/2 - 2; i ++){
    temp = line - table[i];
    if ((temp) > (signed int)(Image->ImageHeight - 1))
      temp = temp - Image->ImageHeight - 1;
    if (temp < 0)
      temp = Image->ImageHeight - 1 + temp;
    DrawPictureLine(Image, temp, xcoo, ycoo);
    line ++;
    ycoo ++;
  }
  for (i = (Image->Height)/2; i >= 0; i --){
    temp = line + table[i];
    if (temp > (signed int)(Image->ImageHeight - 1))
      temp = temp - Image->ImageHeight - 1;
    if (temp < 0)
      temp = Image->ImageHeight - 1 + temp;
    DrawPictureLine(Image, temp, xcoo, ycoo);
    line ++;
    ycoo ++;
  }
}

/*******************************************************************************
* Function Roll_Get_Level(TRoller *Roller)
* ------------------------------------------------------------------------------
* Overview: Function returns current level of desired roller
* Input: pointer to roller
* Output: Nothing
*******************************************************************************/
char Roll_Get_Level(TRoller *Roller){
  Roller->currentlevel = Roller->position / (Roller->ImageHeight / Roller->levels);
  return Roller->currentlevel;
}

/*******************************************************************************
* Function Roll_Set_Level(TRoller *Roller, char level)
* ------------------------------------------------------------------------------
* Overview: Function sets current level and possition of desired roller
* Input: pointer to roller, desired level
* Output: Nothing
*******************************************************************************/
void Roll_Set_Level(TRoller *Roller, char level){
  Roller->currentlevel = level;
  Roller->position = Roller->ImageHeight * level / Roller->levels;
}

/*******************************************************************************
* Function Roll_Roller(TRoller *Roller, char dir, char step)
* ------------------------------------------------------------------------------
* Overview: Function rolls desired roller in desired direction and for desired step
* Input: pointer to roller, direction and step to roll
* Output: Nothing
*******************************************************************************/
static void Roll_Roller(TRoller *Roller, char dir, char step){

  if (dir)
    Roller->position += step;
  else
    Roller->position -= step;

  if (Roller->position > (signed int)Roller->ImageHeight - 1)
    Roller->position = 0;
  if (Roller->position < 0)
    Roller->position = Roller->ImageHeight - 1;
    
  HID_Mouse_Restore_Display();
  Draw_Picture(Roller, Roller->position - Roller->Height/2, Roller->Left, Roller->Top);
  HID_Mouse_Restore_Buffer();
}

/*******************************************************************************
* Function Set_Roller(TRoller *Roller, int pos, char step)
* ------------------------------------------------------------------------------
* Overview: Function sets roller to desired possition by specified step
* Input: pointer to roller, desired possition and step
* Output: Nothing
*******************************************************************************/
static void Set_Roller(TRoller *Roller, int pos, char step){
int temp;
char direction;
  temp = pos * Roller->ImageHeight / Roller->levels + Roller->ImageHeight / Roller->levels / 2;
  temp = temp - Roller->position;
  if (temp > 0)
    direction = 1;
  else{
    direction = 0;
    temp = -temp;
  }
  while (temp > step){
    Roll_Roller(Roller, direction, step);
    temp -= step;
  }
  Roll_Roller(Roller, direction, temp);
}

/*******************************************************************************
* Function Roller_Deactivate(TRoller *Roller, unsigned int xpos, unsigned int ypos)
* ------------------------------------------------------------------------------
* Overview: Function deactivates roller and displays its value instead
* Input: pointer to roller, coordinates where it should display
* Output: Nothing
*******************************************************************************/
static void Roller_Deactivate(TRoller *Roller, unsigned int xpos, unsigned int ypos){
char text[2];
  TFTIntern_Set_Pen(Clock.Color, 0);
  TFTIntern_Set_Brush(1, Clock.Color, 0, 0, 0, 0);
  TFTIntern_Rectangle(Roller->Left, Roller->Top - 2, Roller->Left + Roller->Width, Roller->Top + Roller->Height + 2);
  text[0] = Roller->currentlevel + 48;
  text[1] = 0;
  TFTIntern_Set_Font(Label15.FontName, CL_WHITE_Intern, FO_HORIZONTAL_Intern);
  if (Roller == &AmPm){
    if (Roller->currentlevel == 0)
      TFTIntern_Write_Text("AM", xpos, ypos);
    else if (Roller->currentlevel == 1)
      TFTIntern_Write_Text("PM", xpos, ypos);
  }
  else
    TFTIntern_Write_Text(text, xpos, ypos);
}

/*******************************************************************************
* Function Roller_Init()
* ------------------------------------------------------------------------------
* Overview: Function sets default values for rollers
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void Roller_Init(){
  Hour1Roll.Left = 150;
  Hour1Roll.Top  = 80;
  Hour1Roll.Width = 60;
  Hour1Roll.Height = 340;
  Hour1Roll.ImageHeight = 240;
  Hour1Roll.active = 0;
  Hour1Roll.Picture_Name = stripe_numbers_rtc_bmp;
  Hour1Roll.position = 240 / 2 / 2; // 740 / 10 / 2;
  Hour1Roll.levels = 2;

  Hour2Roll.Left = 233;
  Hour2Roll.Top  = 80;
  Hour2Roll.Width = 60;
  Hour2Roll.Height = 340;
  Hour2Roll.ImageHeight = 1204;
  Hour2Roll.active = 0;
  Hour2Roll.Picture_Name = stripe_numbers_rtc_bmp;
  Hour2Roll.position =  1204 / 10 / 2;
  Hour2Roll.levels = 10;

  Minute1Roll.Left = 328;
  Minute1Roll.Top  = 80;
  Minute1Roll.Width = 60;
  Minute1Roll.Height = 340;
  Minute1Roll.ImageHeight = 720;
  Minute1Roll.active = 0;
  Minute1Roll.Picture_Name = stripe_numbers_rtc_bmp;
  Minute1Roll.position =  720 / 6 / 2;
  Minute1Roll.levels = 6;

  Minute2Roll.Left = 400;
  Minute2Roll.Top  = 80;
  Minute2Roll.Width = 60;
  Minute2Roll.Height = 340;
  Minute2Roll.ImageHeight = 1204;
  Minute2Roll.active = 0;
  Minute2Roll.Picture_Name = stripe_numbers_rtc_bmp;
  Minute2Roll.position =  1204 / 10 / 2;
  Minute2Roll.levels = 10;
  
  AmPm.Left         = 477;
  AmPm.Top          = 152;
  AmPm.Width        = 145;
  AmPm.Height       = 200;
  AmPm.ImageHeight  = 237;
  AmPm.Picture_Name = am_pm_bmp;
  AmPm.currentlevel = 0;
  AmPm.active       = 0;
  AmPm.position     = 237 / 2 / 2;
  AmPm.levels       = 2;
}

/*******************************************************************************
* Function HoursTens()
* ------------------------------------------------------------------------------
* Overview: Function should be run on appropriate roller press
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void HoursTens(){
  // deactivate if any other roller is active
  if (Hour1Roll.active == 0){
    if (Hour2Roll.active == 1){
      Roller_Deactivate(&Hour2Roll, 238, 180);
      Hour2Roll.active = 0;
    }
    if (Minute1Roll.active == 1){
      Roller_Deactivate(&Minute1Roll, 333, 180);
      Minute1Roll.active = 0;
    }
    if (Minute2Roll.active == 1){
      Roller_Deactivate(&Minute2Roll, 405, 180);
      Minute2Roll.active = 0;
    }
    if (AmPm.active == 1){
      Roller_Deactivate(&AmPm, 485, 180);
      AmPm.active = 0;
    }
    // Draw current position
    Draw_Picture(&Hour1Roll, Hour1Roll.position - 96, Hour1Roll.Left, Hour1Roll.Top);
  }
  // run roller
  MyBoxOnPress(&Hour1Roll);
  Hour1Roll.active = 1;
}

/*******************************************************************************
* Function HoursOnes()
* ------------------------------------------------------------------------------
* Overview: Function should be run on appropriate roller press
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void HoursOnes(){
  // deactivate if any other roller is active
  if (Hour2Roll.active == 0){
    if (Hour1Roll.active == 1){
      Roller_Deactivate(&Hour1Roll, 155, 180);
      Hour1Roll.active = 0;
    }
    if (Minute1Roll.active == 1){
      Roller_Deactivate(&Minute1Roll, 333, 180);
      Minute1Roll.active = 0;
    }
    if (Minute2Roll.active == 1){
      Roller_Deactivate(&Minute2Roll, 405, 180);
      Minute2Roll.active = 0;
    }
    if (AmPm.active == 1){
      Roller_Deactivate(&AmPm, 485, 180);
      AmPm.active = 0;
    }
    // Draw current position
    Draw_Picture(&Hour2Roll, Hour2Roll.position - 96, Hour2Roll.Left, Hour2Roll.Top);
  }
  // run roller
  MyBoxOnPress(&Hour2Roll);
  Hour2Roll.active = 1;
}

/*******************************************************************************
* Function MinutesTens()
* ------------------------------------------------------------------------------
* Overview: Function should be run on appropriate roller press
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void MinutesTens(){
  // deactivate if any other roller is active
  if (Minute1Roll.active == 0){
    if (Hour1Roll.active == 1){
      Roller_Deactivate(&Hour1Roll, 155, 180);
      Hour1Roll.active = 0;
    }
    if (Hour2Roll.active == 1){
      Roller_Deactivate(&Hour2Roll, 238, 180);
      Hour2Roll.active = 0;
    }
    if (Minute2Roll.active == 1){
      Roller_Deactivate(&Minute2Roll, 405, 180);
      Minute2Roll.active = 0;
    }
    if (AmPm.active == 1){
      Roller_Deactivate(&AmPm, 485, 180);
      AmPm.active = 0;
    }
    // Draw current position
    Draw_Picture(&Minute1Roll, Minute1Roll.position - 95, Minute1Roll.Left, Minute1Roll.Top);
  }
  // run roller
  MyBoxOnPress(&Minute1Roll);
  Minute1Roll.active = 1;
}

/*******************************************************************************
* Function MinutesOnes()
* ------------------------------------------------------------------------------
* Overview: Function should be run on appropriate roller press
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void MinutesOnes(){
  // deactivate if any other roller is active
  if (Minute2Roll.active == 0){
    if (Hour1Roll.active == 1){
      Roller_Deactivate(&Hour1Roll, 155, 180);
      Hour1Roll.active = 0;
    }
    if (Hour2Roll.active == 1){
      Roller_Deactivate(&Hour2Roll, 238, 180);
      Hour2Roll.active = 0;
    }
    if (Minute1Roll.active == 1){
      Roller_Deactivate(&Minute1Roll, 333, 180);
      Minute1Roll.active = 0;
    }
    if (AmPm.active == 1){
      Roller_Deactivate(&AmPm, 485, 180);
      AmPm.active = 0;
    }
    // Draw current position
    Draw_Picture(&Minute2Roll, Minute2Roll.position - 95, Minute2Roll.Left, Minute2Roll.Top);
  }
  // run roller
  MyBoxOnPress(&Minute2Roll);
  Minute2Roll.active = 1;
}

/*******************************************************************************
* Function AmPmRun()
* ------------------------------------------------------------------------------
* Overview: Function should be run on appropriate roller press
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void AmPmRun(){
  // deactivate if any other roller is active
  if (AmPm.active == 0){
    if (Hour1Roll.active == 1){
      Roller_Deactivate(&Hour1Roll, 155, 180);
      Hour1Roll.active = 0;
    }
    if (Hour2Roll.active == 1){
      Roller_Deactivate(&Hour2Roll, 238, 180);
      Hour2Roll.active = 0;
    }
    if (Minute1Roll.active == 1){
      Roller_Deactivate(&Minute1Roll, 333, 180);
      Minute1Roll.active = 0;
    }
    if (Minute2Roll.active == 1){
      Roller_Deactivate(&Minute2Roll, 405, 180);
      Minute2Roll.active = 0;
    }
    // Draw current position
    Draw_Picture(&AmPm, AmPm.position - 75, AmPm.Left, AmPm.Top);
  }
  // run roller
  MyBoxOnPress(&AmPm);
  AmPm.active = 1;
}

/*******************************************************************************
* Function Deactivate_All_Rollers()
* ------------------------------------------------------------------------------
* Overview: Function deactivates all rollers
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void Deactivate_All_Rollers(){
  if (Hour1Roll.active == 1){
    Roller_Deactivate(&Hour1Roll, 155, 180);
    Hour1Roll.active = 0;
  }
  if (Hour2Roll.active == 1){
    Roller_Deactivate(&Hour2Roll, 238, 180);
    Hour2Roll.active = 0;
  }
  if (Minute1Roll.active == 1){
    Roller_Deactivate(&Minute1Roll, 333, 180);
    Minute1Roll.active = 0;
  }
  if (Minute2Roll.active == 1){
    Roller_Deactivate(&Minute2Roll, 405, 180);
    Minute2Roll.active = 0;
  }
  if (AmPm.active == 1){
    Roller_Deactivate(&AmPm, 485, 180);
    AmPm.active = 0;
  }
}

/*******************************************************************************
* Function RTC_Example_init()
* ------------------------------------------------------------------------------
* Overview: Function initializes RTC example (sets default possitions and values)
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void RTC_Example_init(){
  Roller_Init();
  Ethernet_Message(" ");
  RTCC_Read(&MyTime);
  Roll_Set_Level(&Hour1Roll, MyTime.hours / 10);
  Roll_Set_Level(&Hour2Roll, MyTime.hours % 10);
  Roll_Set_Level(&Minute1Roll, MyTime.minutes / 10);
  Roll_Set_Level(&Minute2Roll, MyTime.minutes % 10);
  Roll_Set_Level(&AmPm, MyTime.ampm);
  Roller_Deactivate(&Hour1Roll, 155, 180);
  Roller_Deactivate(&Hour2Roll, 238, 180);
  Roller_Deactivate(&Minute1Roll, 333, 180);
  Roller_Deactivate(&Minute2Roll, 405, 180);
  Roller_Deactivate(&AmPm, 485, 180);
}

/*******************************************************************************
* Function doRTC_Example()
* ------------------------------------------------------------------------------
* Overview: Function should be called from main while loop when no press is detected
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void doRTC_Example(){
  // reset roll flag
  Roll_Flag = 0;
  // if roller is active and released set it to nearest possition
  if (Hour1Roll.active){
    Roll_Get_Level(&Hour1Roll);
    Set_Roller(&Hour1Roll, Hour1Roll.currentlevel, 1);
  }
  // if roller is active and released set it to nearest possition
  if (Hour2Roll.active){
    Roll_Get_Level(&Hour2Roll);
    Set_Roller(&Hour2Roll, Hour2Roll.currentlevel, 1);
  }
  // if roller is active and released set it to nearest possition
  if (Minute1Roll.active){
    Roll_Get_Level(&Minute1Roll);
    Set_Roller(&Minute1Roll, Minute1Roll.currentlevel, 1);
  }
  // if roller is active and released set it to nearest possition
  if (Minute2Roll.active){
    Roll_Get_Level(&Minute2Roll);
    Set_Roller(&Minute2Roll, Minute2Roll.currentlevel, 1);
  }
  // if roller is active and released set it to nearest possition
  if (AmPm.active){
    Roll_Get_Level(&AmPm);
    Set_Roller(&AmPm, AmPm.currentlevel, 1);
  }
  // check if the new Time is acquaried from Time server
  check_time_server();
}

/*******************************************************************************
* Function Set_MyRTCC()
* ------------------------------------------------------------------------------
* Overview: Function reads current Time and modifyies hours and minutes as set on display
* Input: Nothing
* Output: Nothing
*******************************************************************************/
char Set_MyRTCC(){
char temp;
  RTCC_Read(&MyTime);
  temp = Hour1Roll.currentlevel * 10 + Hour2Roll.currentlevel;
  if (temp > 12)
    return 0;
  MyTime.hours = temp;
  MyTime.minutes = Minute1Roll.currentlevel * 10 + Minute2Roll.currentlevel;
  MyTime.ampm = AmPm.currentlevel;
  Set_RTC(&MyTime);
  return 1;
}

/*******************************************************************************
* Function UpdateTime(TTime *RTCC_Time)
* ------------------------------------------------------------------------------
* Overview: Function reads Time and displays it on Home screen
* Input: Nothing
* Output: Nothing
*******************************************************************************/
static void UpdateTime(TTime *RTCC_Time){
  Label19_Caption[0] = (RTCC_Time->hours / 10) + 48;
  Label19_Caption[1] = (RTCC_Time->hours % 10) + 48;
  Label19_Caption[2] = ':';
  Label19_Caption[3] = (RTCC_Time->minutes / 10) + 48;
  Label19_Caption[4] = (RTCC_Time->minutes % 10) + 48;
  Label19_Caption[5] = ' ';
  if (RTCC_Time->ampm == 0){
    Label19_Caption[6] = 'A';
    Label19_Caption[7] = 'M';
  }
  else if (RTCC_Time->ampm == 1){
    Label19_Caption[6] = 'P';
    Label19_Caption[7] = 'M';
  }
  Label19_Caption[8] = 0;
  TFTIntern_Set_Brush(1, Home.Color, 0, 0, 0, 0);
  TFTIntern_Set_Pen(Home.Color, 0);
  TFTIntern_Rectangle(Label19.Left, Label19.Top, Label19.Left + 85, Label19.Top + 25);
  DrawLabel(&Label19);
}

/*******************************************************************************
* Function checkRTC()
* ------------------------------------------------------------------------------
* Overview: Function checks if Time has changed (for polling purposes)
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void checkRTC(){
  if (RTCC_Read(&MyTime)){
    if ((MyTime.minutes != OldMyTime.minutes) || (MyTime.hours != OldMyTime.hours) || (MyTime.ampm != OldMyTime.ampm)){
      UpdateTime(&MyTime);
      OldMyTime = MyTime;
    }
  }
}

/*******************************************************************************
* Function check_time_server()
* ------------------------------------------------------------------------------
* Overview: Function updates Time which was retreived from Time server
* Input: Nothing
* Output: Nothing
*******************************************************************************/
static void check_time_server(){
  if (Time_Obtained == 1){
    Time_Obtained = 0;
    MyTime.seconds = my_time.ss;
    MyTime.minutes = my_time.mn;
    if (my_time.hh == 0){
      MyTime.ampm  = 0;
      MyTime.hours = 12;
    }
    else if (my_time.hh == 12){
      MyTime.ampm  = 1;
      MyTime.hours = 12;
    }
    else{
      MyTime.hours   = my_time.hh % 12;
      MyTime.ampm    = my_time.hh / 12;
    }
    MyTime.day     = my_time.md;
    MyTime.month   = my_time.mo;
    MyTime.year    = my_time.yy;
    Set_RTC(&MyTime);
    
    Ethernet_Message("Synchronized with GMT");
    Roll_Set_Level(&Hour1Roll, MyTime.hours / 10);
    Roll_Set_Level(&Hour2Roll, MyTime.hours % 10);
    Roll_Set_Level(&Minute1Roll, MyTime.minutes / 10);
    Roll_Set_Level(&Minute2Roll, MyTime.minutes % 10);
    Roll_Set_Level(&AmPm, MyTime.ampm % 10);
    
    Set_Roller(&Hour1Roll, Hour1Roll.currentlevel, 10);
    Roller_Deactivate(&Hour1Roll, 155, 180);
    Set_Roller(&Hour2Roll, Hour1Roll.currentlevel, 10);
    Roller_Deactivate(&Hour2Roll, 238, 180);
    Set_Roller(&Minute1Roll, Hour1Roll.currentlevel, 10);
    Roller_Deactivate(&Minute1Roll, 333, 180);
    Set_Roller(&Minute2Roll, Hour1Roll.currentlevel, 10);
    Roller_Deactivate(&Minute2Roll, 405, 180);
    Set_Roller(&AmPm, AmPm.currentlevel, 10);
    Roller_Deactivate(&AmPm, 485, 180);
  }
}

/*******************************************************************************
* End of File
*******************************************************************************/