/*******************************************************************************
* Scroll  - Function Set used for text scrolling
* ------------------------------------------------------------------------------
*
*******************************************************************************/
#include "mikromedia_5_TIVA_objects.h"
#include "mikromedia_5_TIVA_resources.h"
#include "built_in.h"
#include "Init_routines.h"
#include "HID_Mouse.h"
#include "MP3_main.h"
#include "__Lib_FAT32.h"

// externs
extern unsigned int Xcoord, Ycoord;

// globals
typedef struct Scroll_List {
  char FileName[13];
  char FileType;
} TScroll_List;

char Total_Entries;
TScroll_List File_List[100];


static __DIR MyDir;
static short MyError;

// scrolling constants
static const _MP3_IMAGE_VLINE = 50;
static const _MP3_TEXT_VLINE = 80;
static const _MP3_BIGIMAGE_VLINE = 300;
static const _MP3_BACKGOUND_COLOR = 0x2B31;
static const _MP3_FONT_COLOR = CL_WHITE_Intern;
static const _SCROLL_WIDTH = 200;
static const _SCROLL_HEIGHT = 330;
static const _SCROLL_FIRST_LINE = 80;
static const _SCROLL_LAST_LINE = 410;
static const _FONT_HEIGHT = 25;
static const _FONT_VERTICAL_SPACING = 15;

static const char *MP3_Font = Open_Sans_Light21x33_Regular;

// static
static const _UP = 1;
static const _DOWN = 0;

static int _disp_scroll = 80;
static int imageLine;
static int _disp_scroll_rollover = 0;
static char Scroll_text[15];
static char pix_array[480];
static char current_text_column;
static signed int first_window_column = 0;
static signed int last_window_column = 190;
static unsigned int Scroll_total_columns = 350;
static char current_loaded_text = 0;     
static unsigned int ScrollOldYcoo;
static char Scroll_Flag = 0;

static unsigned int* Temp_Text_Buffer_ptr = 0x10554000;        // (800x40)
static unsigned int* Temp_Scroll_Window_ptr = 0x10564000;      // (200x330)

void Stop_Scroll();
void Initialize_Scroll();

/*******************************************************************************
* Function CopyFromBuffer_ToDisplay()
* ------------------------------------------------------------------------------
* Overview: Function Copies data from buffer in SRAM to Display (addresses in SRAM used for Display)
* Input: Nothing
* Output: Nothing
*******************************************************************************/
static void CopyFromBuffer_ToDisplay(){
  unsigned int i, j, *ptr, *ptr2;
  unsigned long offset;
  
  for (j = 0; j < _SCROLL_HEIGHT - (_disp_scroll - _SCROLL_FIRST_LINE); j ++){
    i = _SCROLL_WIDTH;
    ptr = &TFTIntern_rowData[0];
    ptr2 = Temp_Scroll_Window_ptr + (unsigned long)(j + _disp_scroll - _SCROLL_FIRST_LINE) * _SCROLL_WIDTH;
    while (i --)
      *ptr++ = *ptr2++;
    offset = Get_Cursor_Offset(_MP3_IMAGE_VLINE, j + _SCROLL_FIRST_LINE);
    TFTIntern_Move_Cursor(_MP3_IMAGE_VLINE, j + _SCROLL_FIRST_LINE);
    ptr2 = My_MemoryStartPtr + offset;
    EPI_WorkaroundHWordWriteArrayDec(ptr2, TFTIntern_rowData, _SCROLL_WIDTH);
  }
  for (j = 0; j < _disp_scroll - _SCROLL_FIRST_LINE; j ++){
    i = _SCROLL_WIDTH;
    ptr = &TFTIntern_rowData[0];
    ptr2 = Temp_Scroll_Window_ptr + (unsigned long)(j) * _SCROLL_WIDTH;
    while (i --)
      *ptr++ = *ptr2++;
    offset = Get_Cursor_Offset(_MP3_IMAGE_VLINE, j + _SCROLL_FIRST_LINE + _SCROLL_HEIGHT - (_disp_scroll - _SCROLL_FIRST_LINE));
    TFTIntern_Move_Cursor(_MP3_IMAGE_VLINE, j + _SCROLL_FIRST_LINE + _SCROLL_HEIGHT - (_disp_scroll - _SCROLL_FIRST_LINE));
    ptr2 = My_MemoryStartPtr + offset;
    EPI_WorkaroundHWordWriteArrayDec(ptr2, TFTIntern_rowData, _SCROLL_WIDTH);
  }
}

/*******************************************************************************
* Function CopyFromDisplay_ToBuffer()
* ------------------------------------------------------------------------------
* Overview: Function Copies display content to a buffer in SRAM
* Input: Nothing
* Output: Nothing
*******************************************************************************/
static void CopyFromDisplay_ToBuffer(){
  signed int i, j;
  unsigned *ptr, *ptr2;
  unsigned long offset;

  for (j = _SCROLL_HEIGHT - 0; j >= 0 ; j --){
    i = _SCROLL_WIDTH;
    ptr = &TFTIntern_rowData[0];
    offset = Get_Cursor_Offset(_MP3_IMAGE_VLINE, j + _SCROLL_FIRST_LINE);
    TFTIntern_Move_Cursor(_MP3_IMAGE_VLINE, j + _SCROLL_FIRST_LINE);
    ptr2 = My_MemoryStartPtr + offset;
    while (i --)
      *ptr++ = *ptr2--;
    ptr = Temp_Scroll_Window_ptr + (unsigned long)j * _SCROLL_WIDTH;
    EPI_WorkaroundHWordWriteArrayInc(ptr, TFTIntern_rowData, _SCROLL_WIDTH);
  }
}

/*******************************************************************************
* Function Scroll_Get_One_Text_Line(char line, char *text)
* ------------------------------------------------------------------------------
* Overview: Function Gets desired Pixel Line of text and fills it in a array
* Input: desired line number, pointer to text
* Output: Nothing
*******************************************************************************/
static void Scroll_Get_Text_Line(unsigned int* StartPtr, char *text, char type){
unsigned int *temp, i;

  i = 24000;
  temp = StartPtr;
  while (i--)
    *temp++ = _MP3_BACKGOUND_COLOR;

  TFTIntern_SetDisplayRotation(_TFT_INTERN_ROTATE_0);
  temp = My_MemoryStartPtr;
  TFTIntern_SetMemoryStartAddr(StartPtr);
  TFTIntern_Set_Font(MP3_Font, _MP3_FONT_COLOR, FO_HORIZONTAL_Intern);
  TFTIntern_Write_Text(text, _MP3_TEXT_VLINE - _MP3_IMAGE_VLINE, 0);
  
  if (type == 1)
    TFTIntern_Image(0, 0, file_icon.Picture_Name, 1);
  if (type == 2)
    TFTIntern_Image(0, 0, folder_icon.Picture_Name, 1);
  TFTIntern_SetMemoryStartAddr(temp);
  TFTIntern_SetDisplayRotation(_TFT_INTERN_ROTATE_180);
}

/*******************************************************************************
* Function Scroll_New_Tex_From_List(char text_number, char *Text)
* ------------------------------------------------------------------------------
* Overview:  Function copies one entry from list into desired string
* Input: desired entry, pointer to text
* Output: Nothing
*******************************************************************************/
static char Scroll_New_Tex_From_List(char text_number, char *Text){
char *ptr, *dptr;
  dptr = Text;
  ptr = File_List[text_number].FIleName;

  while (*dptr++ = *ptr++)
    ;
    
  return File_List[text_number].FIleType;
}

/*******************************************************************************
* Function Get_Next_File_Name(signed short number, char *Text)
* ------------------------------------------------------------------------------
* Overview:  Returns type of desired file
* Input: desired entry, pointer to text
* Output: 2 No more files
*         1 audio file (mp3 or wav)
*         0 directory
*******************************************************************************/
char Get_Next_File_Name(signed short number, char *Text){
char temp;
  if ((number > Total_Entries) || (number < 0))
    return 2;
  temp = Scroll_New_Tex_From_List(number, Text);
  if (temp == 1)
    return 1;
  else if (temp == 2)
    return 0;
}

/*******************************************************************************
* Function Scrool_Get_new_text(char Text_no)
* ------------------------------------------------------------------------------
* Overview:  Function gets new text line for displaying into temp buffer
* Input: desired entry
* Output: File type
*******************************************************************************/
static char Scrool_Get_new_text(char Text_no){
char FType;
  if (current_loaded_text != Text_no){
    current_loaded_text = Text_no;
    FType = Scroll_New_Tex_From_List(Text_no, Scroll_Text);
    Scroll_Get_Text_Line(Temp_Text_Buffer_ptr, Scroll_Text, FType);
  }
  return File_List[Text_no].FileType;
}

/*******************************************************************************
* Function Scroll_Add_One_Line(unsigned int HO_Line, char row)
* ------------------------------------------------------------------------------
* Overview:  Function writes one horizontal line from Temp_text buffer  to Temp_Scroll_Window
* Input: desired line of text, desired Line on Scroll window
* Output: Nothing
*******************************************************************************/
static void Scroll_Add_One_Line(unsigned int HO_Line, char row){
  unsigned int *ptr_buffer, *ptr_text;
  char i;
  
  ptr_text = Temp_Text_Buffer_ptr + row * 800;
  ptr_buffer =  TFTIntern_rowData;
  i = _SCROLL_WIDTH;
  while (i--)
    *ptr_buffer++ = *ptr_text++;

  ptr_buffer = Temp_Scroll_Window_ptr + (unsigned long)HO_Line * _SCROLL_WIDTH;
  EPI_WorkaroundHWordWriteArrayInc(ptr_buffer, TFTIntern_rowData, _SCROLL_WIDTH);
}

/*******************************************************************************
* Function Scroll_Fill_Area()
* ------------------------------------------------------------------------------
* Overview:  Function fills Scroll area with text from File_List
* Input: Nothing
* Output: Nothing
*******************************************************************************/
static void Scroll_Fill_Area(){
char *ptr, *dptr, i;
unsigned int temp, temp1;
  dptr = Scroll_text;
  i = 0;
  ptr = File_List[i].FileName;

  // write Text
  while (*ptr != 0xFF){
    while(*dptr++ = *ptr++)
      ;
    temp = i * (_FONT_HEIGHT + _FONT_VERTICAL_SPACING);
    ptr = File_List[i+1].FileName;
    if (temp < _SCROLL_LAST_LINE - _SCROLL_FIRST_LINE - _FONT_HEIGHT){
      if (File_List[i].FileType == 1)
        TFTIntern_Image(_MP3_IMAGE_VLINE, _SCROLL_FIRST_LINE + temp, file_icon.Picture_Name, 1);
      if (File_List[i].FileType == 2)
        TFTIntern_Image(_MP3_IMAGE_VLINE, _SCROLL_FIRST_LINE + temp, folder_icon.Picture_Name, 1);
      TFTIntern_Set_Font(MP3_Font, _MP3_FONT_COLOR, FO_HORIZONTAL_Intern);
      TFTIntern_Write_Text(Scroll_text, _MP3_TEXT_VLINE, _SCROLL_FIRST_LINE + temp);
      i ++;
      dptr = Scroll_text;
      temp1 = _SCROLL_FIRST_LINE + temp;
    }
    else{
      i ++;
      dptr = Scroll_text;
    }
  }
  Scroll_total_columns = temp + (_FONT_HEIGHT + _FONT_VERTICAL_SPACING);
  CopyFromDisplay_ToBuffer();

  // if there is uncomplete text, write it too
  temp1 = temp1 + _FONT_HEIGHT + _FONT_VERTICAL_SPACING - 1;
  if (temp1 < _SCROLL_LAST_LINE)
    current_text_column = 1;
  if (temp1 <= temp){
    while (temp1 < _SCROLL_LAST_LINE){
      i = Scrool_Get_new_text(last_window_column / (_FONT_HEIGHT + _FONT_VERTICAL_SPACING));

      if (File_List[i].FileType == 1)
        Scroll_Add_One_Line(temp1 - _SCROLL_FIRST_LINE, current_text_column);
      if (File_List[i].FileType == 2)
        Scroll_Add_One_Line(temp1 - _SCROLL_FIRST_LINE, current_text_column);

      current_text_column ++;
      temp1 ++;
    }
    CopyFromBuffer_ToDisplay();
  }
}

/*******************************************************************************
* Function Scroll_Add_newTextLine(char ScrollDiection)
* ------------------------------------------------------------------------------
* Overview:  Function adds new line of text after scrolling
* Input: Scroll direction
* Output: Nothing
*******************************************************************************/
static void Scroll_Add_newTextLine(char ScrollDiection){
unsigned int temp;
char temp_file_type;

  if (ScrollDiection == 0){
    current_text_column = last_window_column % (_FONT_HEIGHT + _FONT_VERTICAL_SPACING);
    if (current_text_column < _FONT_HEIGHT + 1){
      temp_file_type = Scrool_Get_new_text(last_window_column / (_FONT_HEIGHT + _FONT_VERTICAL_SPACING));
      
      temp = _disp_scroll - 1;
      if (temp < _SCROLL_FIRST_LINE)
        temp = _SCROLL_LAST_LINE - 1;
      
      Scroll_Add_One_Line(temp - _SCROLL_FIRST_LINE, current_text_column);
    }
    else {
      temp = _disp_scroll - 1;
      if (temp < _SCROLL_FIRST_LINE)
        temp = _SCROLL_LAST_LINE - 1;
        Scroll_Add_One_Line(temp - _SCROLL_FIRST_LINE, 29);
    }
  }
  else{
    current_text_column = first_window_column % (_FONT_HEIGHT + _FONT_VERTICAL_SPACING);
    if (current_text_column < _FONT_HEIGHT + 1){
      temp_file_type = Scrool_Get_new_text(first_window_column / (_FONT_HEIGHT + _FONT_VERTICAL_SPACING));
      Scroll_Add_One_Line(_disp_scroll - _SCROLL_FIRST_LINE, current_text_column);
    }
    else
      Scroll_Add_One_Line(_disp_scroll  - _SCROLL_FIRST_LINE, 29);
  }
}

/*******************************************************************************
* Function Scroll_Menu(char Adir)
* ------------------------------------------------------------------------------
* Overview:  Function scrolls display in desired directon
* Input: Scroll direction
* Output: Nothing
*******************************************************************************/
static void Scroll_Menu(char Adir) {

  if (Adir == _Up) {
      if (_disp_scroll != _SCROLL_FIRST_LINE)
        _disp_scroll--;
      else {
        _disp_scroll = _SCROLL_LAST_LINE - 1;
        _disp_scroll_rollover --;
      }
  }
  else {
      if (_disp_scroll != _SCROLL_LAST_LINE - 1)
        _disp_scroll++;
      else {
        _disp_scroll = _SCROLL_FIRST_LINE;
        _disp_scroll_rollover ++;
      }
  }
}

/*******************************************************************************
* Function ScrollUp(int scroll)
* ------------------------------------------------------------------------------
* Overview:  Function should be called to perform Scroll up
* Input: number of lines to scroll
* Output: Nothing
*******************************************************************************/
static void ScrollUp(int scroll){
  while(scroll--){
    last_window_column --;
    first_window_column --;
    if (first_window_column >= 0){

      Scroll_Menu(_Up);              // Scroll image
      Scroll_Add_newTextLine(_Up);
    }
    else {
      last_window_column ++;
      first_window_column ++;
    }
  }
}

/*******************************************************************************
* Function ScrollDown(int scroll)
* ------------------------------------------------------------------------------
* Overview:  Function should be called to perform Scroll down
* Input: number of lines to scroll
* Output: Nothing
*******************************************************************************/
static void ScrollDown(int scroll){
  while(scroll--){
    last_window_column ++;
    first_window_column ++;
    if (last_window_column < Scroll_total_columns){
    
      Scroll_Menu(_Down);
      Scroll_Add_newTextLine(_Down);
    }
    else{
      last_window_column --;
      first_window_column --;
    }
  }
}

/*******************************************************************************
* Function ScrollDown(unsigned int Apos, char Adir)
* ------------------------------------------------------------------------------
* Overview:  Function should be called to perform Scroll in desired direction
* Input: Scroll direction, number of lines to scroll
* Output: Nothing
*******************************************************************************/
static void Scroll_Disp(unsigned int Apos, char Adir) {
  if (Adir == _Up) {
    ScrollUP(Apos);
  }
  else {
    ScrollDown(Apos);
  }
  CopyFromBuffer_ToDisplay();
}

/*******************************************************************************
* Function ScrollPress()
* ------------------------------------------------------------------------------
* Overview:  Function polls touch panel values. Should be called frequently
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void ScrollPress(){
  int delta;
  char dir;

  if (Xcoord > 240){
    if (Scroll_Flag == 0){
      Scroll_Flag = 1;
      ScrollOldYcoo = Ycoord;
      return;
    }
    delta = Ycoord - ScrollOldYcoo;
    if (delta > 0){
      dir = 0;
    }
    if (delta < 0){
      dir = 1;
      delta = - delta;
    }
    if (delta < 3)
      return;
    if (delta > 5)
      delta = 5;

    if (dir == 1)
      Scroll_Disp(delta, _DOWN);
    else
      Scroll_Disp(delta, _UP);

    ScrollOldYcoo = Ycoord;
  }
}

/*******************************************************************************
* Function Process_file()
* ------------------------------------------------------------------------------
* Overview:  Function process selected file
* Input: Nothing
* Output: Nothing
*******************************************************************************/
static void Process_file(char File_No){
  BLED_Fade_Out();
  Stop_Scroll();
  Start_MP3_Player(File_List[File_No].FileName, File_no);
  BLED_Fade_In();
}

/*******************************************************************************
* Function Process_folder()
* ------------------------------------------------------------------------------
* Overview:  Function process selected folder
* Input: Nothing
* Output: Nothing
*******************************************************************************/
static void Process_folder(char File_No){
  SPI_Set_Active(SPI0_Read, SPI0_Write);
  FAT32_ChangeDir(File_List[File_No].FileName);
  DrawBox(&Box6);
  Initialize_Scroll();
  SPI_Set_Active(SPI1_Read, SPI1_Write);
}

/*******************************************************************************
* Function ScrollOnClick()
* ------------------------------------------------------------------------------
* Overview:  Function determines whish entry was pressed
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void ScrollOnClick(){
char temp, file_type;
  if (Xcoord < 241){
    temp = (Ycoord - _SCROLL_FIRST_LINE + _disp_scroll - _SCROLL_FIRST_LINE + _disp_scroll_rollover * (_SCROLL_LAST_LINE - _SCROLL_FIRST_LINE)) / (_FONT_HEIGHT + _FONT_VERTICAL_SPACING);
    file_type = Scrool_Get_new_text(temp);

    if (file_type == 1)
      Process_file(temp);
    if (file_type == 2)
      Process_folder(temp);
  }
}

/*******************************************************************************
* Function Fill_List(TScroll_List *List)
* ------------------------------------------------------------------------------
* Overview:  Function fills File_List with entries from SD card (FAT32)
* Input: Pointer to File_List
* Output: Total Entries
*******************************************************************************/
static char Fill_List(TScroll_List *List){
char MyError = 1;
char File_List_cnt = 0;

  SPI_Set_Active(SPI0_Read, SPI0_Write);
  memset((char *)List, 0, 100 * (13 + 1));
  FAT32_FindFirst( &MyDir );
  while ((MyError != 0) && (MyError != 0xFF)){
    if ((MyDir.NameExt[0] != 0) && MyError != 0){
      if (MyDir.Attrib & ATTR_FILE_MASK){
        if ((strstr(MyDir.NameExt, ".MP3") != 0) || (strstr(MyDir.NameExt, ".WAV") != 0)){
          strcat(List->FileName, MyDir.NameExt);
          List->FileType = 1;
          File_List_cnt ++;
          List ++;
        }
      }
      else if ((MyDir.Attrib == ATTR_DIRECTORY)  && (strcmp(MyDir.NameExt, ".") != 0)){
        strcat(List->FileName, MyDir.NameExt);
        List->FileType = 2;
        File_List_cnt ++;
        List ++;
      }
    }
    MyError = FAT32_FindNext(&MyDir);
  }
  List->FileName[0] = 0xFF;
  
  SPI_Set_Active(SPI1_Read, SPI1_Write);
  return File_List_cnt;
}

/*******************************************************************************
* Function Stop_Scroll()
* ------------------------------------------------------------------------------
* Overview:  Function stops scroller and resets variables
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void Stop_Scroll(){
  _disp_scroll = _SCROLL_FIRST_LINE;
  _disp_scroll_rollover = 0;
  first_window_column = 0;
  last_window_column = _SCROLL_HEIGHT;
}

/*******************************************************************************
* Function Initialize_Scroll()
* ------------------------------------------------------------------------------
* Overview:  Function initializes scroller and resets variables
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void Initialize_Scroll(){
  SPI_Set_Active(SPI0_Read, SPI0_Write);
  Stop_Scroll();
  FAT32_FindFirst(&MyDir);
  Total_Entries = Fill_List(File_List);
  Scroll_Fill_Area();

  TFTIntern_Image(450, 130, MP3_Image1_SRAM_ptr, 1);
  
  SPI_Set_Active(SPI1_Read, SPI1_Write);
}

/*******************************************************************************
* End of File
*******************************************************************************/