/*******************************************************************************
* Tic Tac Toe example - Function Set used for Tic Tac Toe game
* ------------------------------------------------------------------------------
* Simple demonstration of Tic Tac Toe game.
* Game can be played on one mikromedia as well as on two (using RF module)
*******************************************************************************/
#include "mikromedia_5_TIVA_objects.h"
#include "mikromedia_5_TIVA_resources.h"
#include "built_in.h"
#include "Init_routines.h"
#include "HID_Mouse.h"
#include "__Lib_nRF24L01.h"
#include "nRF24L01_defs.h"

// bRF externals
TnRF_Status nRFStatus;
TRF_Config rfConfig;
unsigned char rfState;

// globals
unsigned int * const TicTacToe_Table_Image_SRAM_ptr = 0x10192800;
char TTT_flag;
char TicTacToe_Table_Loaded = 0;
unsigned int TicTacToe_seconds_cnt;
TButton *ButtonArray[9] = {&Button1, &Button2, &Button3, &Button4, &Button5, &Button6, &Button7, &Button8, &Button9};


// static
static const RF_Stop = 1;
static const unsigned int O_color = 0xDB27;
static const unsigned int X_color = 0x35E8;

// game messages
static const char OsTurn_cap[]   = "O's turn";
static const char XsTurn_cap[]   = "X's turn";
static const char Draw_cap[]     = "Draw";
static const char XWin_cap[]     = "X wins!";
static const char OWin_cap[]     = "O wins!";
static const char YouWin_cap[]   = "Winner!";
static const char YouLose_cap[]  = "Looser!";
static const char XO_X_caption[] = "X";
static const char XO_O_caption[] = "O";

static char RF_used;
static char victory = 0;
static char Myturn = 0;
static char RF_cnt = 0, RF_send_Failed = 0;
static char TicTacToe_Counter = 0;

// game matrix        i  j
static char XO_matrix[3][3] = {{0, 0, 0},        // 0 - empty
                               {0, 0, 0},        // 1 = X
                               {0, 0, 0}};       // 2 - O
static char XO_Turn = 1;     // 1 - X, 2 - O

static char XO_ID = 2, XO_First_Move = 1;
static char XO_ID2 = 2;
static TLabel *LabelArray[9] = {&XO1, &XO2, &XO3, &XO4, &XO5, &XO6, &XO7, &XO8, &XO9};

// package truct
struct Tpacket {
  char ID;
  char first_move;
  char Mat[3][3];
  char turn;
} packet;

// for library use
void Timer_16us_Disable(){
  TIMER_ICR_TATOCINT_TIMER2_ICR_bit = 1;
  NVIC_IntDisable(IVT_INT_TIMER2A_16_32_bit);
}

void Timer_16us_Enable(){
  NVIC_IntEnable(IVT_INT_TIMER2A_16_32_bit);
}

/*******************************************************************************
* Interrupt initialiation code
* ------------------------------------------------------------------------------
* Timer 5: Timer5 (32bit) Prescaler : 0
* Preload = 120000000
* Actual Interrupt Time = 1s
*******************************************************************************/
static void InitTimer3A(){
  SYSCTL_RCGCTIMER_R3_bit = 1;
  EnableInterrupts();
  TIMER_CTL_TAEN_TIMER3_CTL_bit = 0;
  TIMER3_CFG   = 0;
  TIMER3_TAMR |= 2;
  TIMER3_TAILR = 120000000;
//  NVIC_IntEnable(IVT_INT_TIMER3A_16_32_bit);
  TIMER_IMR_TATOIM_TIMER3_IMR_bit = 1;
  TIMER_CTL_TAEN_TIMER3_CTL_bit = 1;
}

/*******************************************************************************
* Interrupt initialiation code
* ------------------------------------------------------------------------------
* Timer 2: Timer2A (16bit) Prescaler : 0
* Preload = 1920
* Actual Interrupt Time = 16us
*******************************************************************************/
static void InitTimer2A(){
  SYSCTL_RCGCTIMER_R2_bit = 1;
  EnableInterrupts();
  TIMER_CTL_TAEN_TIMER2_CTL_bit = 0;
  TIMER2_CFG   = 4;
  TIMER2_TAMR |= 2;
  TIMER2_TAPR  = 0;
  TIMER2_TAILR = 1920;
  NVIC_IntDisable(IVT_INT_TIMER2A_16_32_bit);
  TIMER_IMR_TATOIM_TIMER2_IMR_bit = 1;
  TIMER_CTL_TAEN_TIMER2_CTL_bit = 1;
}

/*******************************************************************************
* Interrupt initialiation code
* ------------------------------------------------------------------------------
* EXTI10: External interrupt on PG10
* Falling edge
*******************************************************************************/
static void InitExt(){
  GPIO_PORTG_AHB_AFSEL |= _GPIO_PINMASK_2;   // Enable alternate pin function
  GPIO_PORTG_AHB_IS    = 0;                 // Edge sensitive
  GPIO_PORTG_AHB_IBE   = 0;
  GPIO_PORTG_AHB_IEV   = 0;                 // Falling edge will trigger the event
  GPIO_PORTG_AHB_IM    = _GPIO_PINMASK_2;   // Set mask to PG7
  NVIC_IntEnable(IVT_INT_GPIOG);
}

/*******************************************************************************
* Function InitRF()
* ------------------------------------------------------------------------------
* Overview: Function initialize appropriate interrupts
* Input: Nothing
* Output: Nothing
*******************************************************************************/
static void InitRF(){

  nRF_CE = 0;
  nRF_CS = 1;

  InitTimer2A();
  InitExt();

  // Initialize SPI1 module
  SPI1_Init_Advanced(5000000, _SPI_MASTER, _SPI_8_BIT | _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION, &_GPIO_MODULE_SPI1_B5E45_AHB);
}

/*******************************************************************************
* Function Start_nRF()
* ------------------------------------------------------------------------------
* Overview: Function starts nRF module and sets flags for game
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void Start_nRF(){
  //nRF Config Settings
  rfConfig.autoTransmitDelay = nRF_AUTO_RETRANSMIT_DELAY_4000us;
  rfConfig.autoTransmitCount = nRF_AUTO_RETRANSMIT_COUNT_05;
  rfConfig.channel           = 64;
  rfConfig.outputPower       = nRF_OUTPUT_POWER_0dBm;
  rfConfig.dataRate          = nRF_DATA_RATE_1Mbps;

  // init module
  nRF_Init();
  InitRF();
  Delay_ms(100);

  TTT_flag = 0;
  rfState = RF_STOP;
  // configure module as RX
  nRF_RX_Mode_Init_Advanced(rfConfig.dataRate, rfConfig.outputPower, rfConfig.channel);
  
  // configure Timer 3A and set invert flag
  InitTimer3A();
  NVIC_IntDisable(IVT_INT_TIMER3A_16_32_bit);
  
  if (RF_used == 1)
    RF_used = 0;
  else
    RF_used = 1;
}

void XO_Reset();
static void UpdateLabel(TLabel * Label, char xo);
static void UpdateStatus(char xo);
static void TestDraw();
static void Test_Victory();
static void XO_Process_Packet(char *receved_packet);
static void RF_Send_Move();
static char Check_For_Draw();

/*******************************************************************************
* Functions used for Buzzer
* ------------------------------------------------------------------------------
* Overview: Simple tones that follows every move.
* Input: Nothing
* Output: Nothing
*******************************************************************************/
static void ToneA() {
  Sound_Play( 880, 50);
}
static void ToneC() {
  Sound_Play(1046, 50);
}
static void ToneE() {
  Sound_Play(1318, 50);
}

static void LED_Move(){
  Sound_Play(1318, 50);
  Sound_Play(1567, 50);
}

static void LED_Victory_Blink(){
char i;
  for (i = 0; i < 5; i ++){
    LED_R = 1;
    ToneA();
    Delay_ms(20);
    LED_R = 0;
    LED_G = 1;
    ToneC();
    Delay_ms(20);
    LED_G = 0;
    LED_B = 1;
    ToneE();
    Delay_ms(20);
  }
  LED_B = 0;
}

/*******************************************************************************
* Function doTicTacToeISR()
* ------------------------------------------------------------------------------
* Overview: updates game duration Label (should be called from 1s interrupt)
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void doTicTacToeISR(){
  Label39_Caption[6] = (TicTacToe_seconds_cnt / 60) / 10 + 48;
  Label39_Caption[7] = (TicTacToe_seconds_cnt / 60) % 10 + 48;
  Label39_Caption[9] = (TicTacToe_seconds_cnt % 60) / 10 + 48;
  Label39_Caption[10] = (TicTacToe_seconds_cnt % 60) % 10 + 48;
  
  HID_Mouse_Restore_Display();
  RedrawLabel(Box8.Color, &Label39, 160, 35);
  HID_Mouse_Restore_Buffer();
}

/*******************************************************************************
* Function UpdateLabel38(char *text, unsigned int color)
* ------------------------------------------------------------------------------
* Overview: Function updates game Status
* Input: desired text, text color
* Output: Nothing
*******************************************************************************/
static void UpdateLabel38(char *text, unsigned int color){
  Label38.Font_Color = color;
  Label38.Caption = text;
  RedrawLabel(Box8.Color, &Label38, 200, 100);
}

/*******************************************************************************
* Function doTicTacToe()
* ------------------------------------------------------------------------------
* Overview: Polls if there is incomming messages from RF (should be called frequently)
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void doTicTacToe(){
  
  TicTacToe_Counter = 0;
  // update game duration (flag is set in interrupt)
  if (TTT_flag){
    doTicTacToeISR();
    TTT_flag = 0;
  }
  // if we play agains RF oponent ...
  if (RF_used == 1){
    // process packet
    nRF_Process(&nRFStatus);

    // if data received process package
    if (nRFStatus._status == nRF_RX_DR){
      LED_B = 1;
      XO_Process_Packet(RX_pload);
      LED_B = 0;
    }

    // if successfully sent, go to RX mode
    if (nRFStatus._status == nRF_TX_DS){
      RF_send_Failed = 0;
      nRF_RX_Mode_Init(rfConfig.channel);
    }

    // if previous send failed, repeat it
    if ((nRFStatus._status == nRF_MAX_RT) || RF_send_Failed){
      RF_send_Failed = 1;
      if (RF_cnt++ > 200){
        LED_B = 1;
        RF_Send_Move();
        Delay_ms(50);

        RF_cnt = 0;
        LED_B = 0;
      }
    }
  }
}

/*******************************************************************************
* Function RF_Send_Move()
* ------------------------------------------------------------------------------
* Overview: Function sends packet over RF
* Input: Nothing
* Output: Nothing
*******************************************************************************/
static void RF_Send_Move(){
char i;
char *pMat;

  nRF_TX_Mode_Init(rfConfig.channel);
  pMat = (char*)XO_matrix;
  TX_pload[0] = XO_ID;
  TX_pload[1] = XO_First_Move;
  for (i = 0; i < 9; i ++)
    TX_pload[i + 2] = *pMat ++;
  TX_pload[11] = XO_Turn;

  nRF_Send_Packet();
  Delay_ms(10);
}

/*******************************************************************************
* Function XO_Process(char No)
* ------------------------------------------------------------------------------
* Overview: Function processes new move
* Input: Number of the received character
* Output: Nothing
*******************************************************************************/
static void XO_Process(char No){
char temp_i, temp_j;

  temp_i = No / 3;
  temp_j = No % 3;

  HID_Mouse_Restore_Display();
  if (victory == 0){
    if (XO_matrix[temp_i][temp_j] == 2){
      UpdateLabel(LabelArray[No], 1);
      UpdateStatus(1);
    }
    else if (XO_matrix[temp_i][temp_j] == 1){
      UpdateLabel(LabelArray[No], 0);
      UpdateStatus(0);
    }
  }
  HID_Mouse_Restore_Buffer();
}

/*******************************************************************************
* Function XO_Process_Packet(char *receved_packet)
* ------------------------------------------------------------------------------
* Overview: Function processes receuived package
* Input: pointer to packet
* Output: Nothing
*******************************************************************************/
static void XO_Process_Packet(char *receved_packet){
char *ptr1;
char *ptr2;
char i;

  ptr1 = (char*)&packet;
  // copy received packet into appropriate structure field
  for (i = 0; i < 12; i ++)
    *ptr1 ++ = *receved_packet ++;

  if (packet.ID != 0){
    ptr1 = (char*)&packet;
    ptr1 += 2;
    ptr2 = (char*)XO_matrix;

    if (packet.ID == XO_ID2){
      if (packet.turn == 0xFF){
        XO_Reset();
        return;
      }
      XO_Turn = packet.turn;
      if (Myturn == 0){
        MyTurn = XO_Turn;
          NVIC_IntEnable(IVT_INT_TIMER3A_16_32_bit);
      }

      for (i = 0; i < 9; i ++){
        if (*ptr1 != *ptr2){
          *ptr2 = *ptr1;
          XO_Process(i);
          LED_G = 1;
          LED_R = 0;
        }
        ptr1 ++;
        ptr2 ++;
      }
      Test_Victory();
      TestDraw();
    }
  }
}

/*******************************************************************************
* Function TestDraw()
* ------------------------------------------------------------------------------
* Overview: Function processes drawn game
* Input: Nothing
* Output: Nothing
*******************************************************************************/
static void TestDraw(){
  if (victory == 0)
    if (Check_For_Draw() == 1){
      NVIC_IntDisable(IVT_INT_TIMER3A_16_32_bit);
      HID_Mouse_Restore_Display();
      UpdateLabel38(Draw_cap, CL_WHITE_Intern);
      HID_Mouse_Restore_Buffer();
      LED_G = 0;
      LED_B = 0;
      LED_R = 0;
    }
}

/*******************************************************************************
* Function XO_Set_New_State(char MatNo)
* ------------------------------------------------------------------------------
* Overview: Function Sets New State for the new move
* Input: matrix position
* Output: 1 if X played move
*         2 if O played move
*         0 if invalid move
*******************************************************************************/
static char XO_Set_New_State(char MatNo){
char temp_i, temp_j;

  temp_i = MatNo / 3;
  temp_j = MatNo % 3;
  if (XO_matrix[temp_i][temp_j] == 0){
     if (XO_Turn == 1){
       XO_matrix[temp_i][temp_j] = 1;
       XO_Turn = 2;
       return 1;
     }
     else if (XO_Turn == 2){
       XO_matrix[temp_i][temp_j] = 2;
       XO_Turn = 1;
       return 2;
     }
  }
  else {
    return 0;
  }
}

/*******************************************************************************
* Function XO_Victory(char victory_flag)
* ------------------------------------------------------------------------------
* Overview: Function marks victory
* Input: victory flag
* Output: Nothing
*******************************************************************************/
static void XO_Victory(char victory_flag){
  switch (victory_flag){
    case 1 : {
              TFTIntern_Set_Pen(CL_WHITE_Intern, 2);
              TFTIntern_Line(388, 126, 733, 114);
             }; break;
    case 2 : {
              TFTIntern_Set_Pen(CL_WHITE_Intern, 2);
              TFTIntern_Line(393, 250, 740, 237);
             }; break;
    case 3 : {
              TFTIntern_Set_Pen(CL_WHITE_Intern, 2);
              TFTIntern_Line(400, 374, 743, 360);
             }; break;
    case 4 : {
              TFTIntern_Set_Pen(CL_WHITE_Intern, 2);
              TFTIntern_Line(435, 85, 450, 410);
             }; break;
    case 5 : {
              TFTIntern_Set_Pen(CL_WHITE_Intern, 2);
              TFTIntern_Line(557, 79, 574, 406);
             }; break;
    case 6 : {
              TFTIntern_Set_Pen(CL_WHITE_Intern, 2);
              TFTIntern_Line(682, 74, 701, 404);
             }; break;
    case 7 : {
              TFTIntern_Set_Pen(CL_WHITE_Intern, 2);
              TFTIntern_Line(393, 86, 740, 398);
             }; break;
    case 8 : {
              TFTIntern_Set_Pen(CL_WHITE_Intern, 2);
              TFTIntern_Line(727, 72, 404, 412);
             }; break;
  }
}

/*******************************************************************************
* Function Check_For_Draw()
* ------------------------------------------------------------------------------
* Overview: Function checks if the there is draw condition
* Input: Nothing
* Output: 1 if game is draw
*         0 if game is not finished yet
*******************************************************************************/
static char Check_For_Draw(){
  char i, j;
  for (i = 0; i < 3; i ++){
    for (j = 0; j < 3; j ++){
      if (XO_matrix[i][j] == 0)
        return 0;
    }
  }
  return 1;
}

/*******************************************************************************
* Function XO_Check_For_Victory()
* ------------------------------------------------------------------------------
* Overview: Function checks if the there is victory condition
* Input: Nothing
* Output: victory flag which is different in all 8 possible scenarios
*******************************************************************************/
static char XO_Check_For_Victory(){
char temp_i, temp_j;

  for (temp_i = 0; temp_i < 3; temp_i ++){
    if (XO_matrix[temp_i][0] != 0)
      if (XO_matrix[temp_i][0] == XO_matrix[temp_i][1])
        if (XO_matrix[temp_i][1] == XO_matrix[temp_i][2])
          return (temp_i + 1);
  }
  for (temp_j = 0; temp_j < 3; temp_j ++){
    if (XO_matrix[0][temp_j] != 0)
      if (XO_matrix[0][temp_j] == XO_matrix[1][temp_j])
        if (XO_matrix[1][temp_j] == XO_matrix[2][temp_j])
          return (temp_j + 4);
  }
  if (XO_matrix[0][0] != 0)
    if (XO_matrix[0][0] == XO_matrix[1][1])
      if (XO_matrix[1][1] == XO_matrix[2][2]){
        return 7;
      }
  if (XO_matrix[0][2] != 0)
    if (XO_matrix[0][2] == XO_matrix[1][1])
      if (XO_matrix[1][1] == XO_matrix[2][0]){
        return 8;
      }
  return 0;
}

/*******************************************************************************
* Function UpdateLabel(TLabel * Label, char xo)
* ------------------------------------------------------------------------------
* Overview: Function updates lable in game table
* Input: pointer to Label, X or O
* Output: Nothing
*******************************************************************************/
static void UpdateLabel(TLabel * Label, char xo){
  if (xo){
    Label->Caption = XO_O_caption;
    Label->Font_Color = O_color;
    DrawLabel(Label);
  }
  else{
    Label->Caption = XO_X_caption;
    Label->Font_Color = X_color;
    DrawLabel(Label);
  }
}

/*******************************************************************************
* Function UpdateStatus(char xo)
* ------------------------------------------------------------------------------
* Overview: Function updates player turn
* Input: X or O
* Output: Nothing
*******************************************************************************/
static void UpdateStatus (char xo){
  if (xo){
    UpdateLabel38(XsTurn_cap, X_color);
  }
  else {
    UpdateLabel38(OsTurn_cap, O_color);
  }
}

/*******************************************************************************
* Function Test_Victory()
* ------------------------------------------------------------------------------
* Overview: Test if there were victory and displays it if so.
* Input: Nothing
* Output: Nothing
*******************************************************************************/
static void Test_Victory(){
  victory = XO_Check_For_Victory();
  if (victory){
    NVIC_IntDisable(IVT_INT_TIMER3A_16_32_bit);
    XO_Victory(victory);
    
    HID_Mouse_Restore_Display();
    if (RF_used){
      if (XO_turn == MyTurn)
        UpdateLabel38(YouLose_cap, CL_BLUE_Intern);
      else
        UpdateLabel38(YouWin_cap, CL_RED_Intern);
    }
    else{
      if (XO_turn == 2)
        UpdateLabel38(XWin_cap, X_color);
      else if (Xo_turn == 1)
        UpdateLabel38(Owin_cap, O_color);
    }
    HID_Mouse_Restore_Buffer();
    
    LED_Victory_Blink();
  }
  else
    LED_Move();
}

/*******************************************************************************
* Function XO_Reset()
* ------------------------------------------------------------------------------
* Overview: Function reset game into default state
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void XO_Reset(){
  // fill matrix with 0 (default state)
  memset(XO_matrix, 0, 9);
  victory = 0;
  XO_Turn = 0xFF;

  // disable second counter
  InitTimer3A();
  NVIC_IntDisable(IVT_INT_TIMER3A_16_32_bit);
  TicTacToe_seconds_cnt = 0;
  doTicTacToeISR();
  
  if (RF_used){
    RF_Send_Move();
    Delay_ms(50);
    nRF_RX_Mode_Init(rfConfig.channel);
  }
  XO_Turn = 1;
  XO_First_Move = 1;
  MyTurn = 0;
  LED_B = 0;
  LED_R = 0;
  LED_G = 0;
  
  HID_Mouse_Restore_Display();
  // redraw game table
  if (TicTacToe_Table_Loaded){                                // from SD card
    TFTIntern_Image(365, 40, (char *)TicTacToe_Table_Image_SRAM_ptr, 1);
  }
  else
    if (Init_FAT()){                                          // displays background picture
      CopyImage_FromResources_ToSRAM(game_table_bmp, TicTacToe_Table_Image_SRAM_ptr);
      TFTIntern_Image(365, 40, (char *)TicTacToe_Table_Image_SRAM_ptr, 1);
      TicTacToe_Table_Loaded = 1;
    }
    
  Label38.Caption = Label38_Caption;
  ReDrawLabel(Box8.Color, &Label38, 200, 100);
  HID_Mouse_Restore_Buffer();
  
  // init RGB
  GPIO_Digital_Output(&GPIO_PORTD_AHB, _GPIO_PINMASK_2 | _GPIO_PINMASK_3 | _GPIO_PINMASK_5);
  LED_B = 0;
  LED_R = 0;
  LED_G = 0;
}

/*******************************************************************************
* Function ButtonClick(TButton *Butt, char No)
* ------------------------------------------------------------------------------
* Overview: Function processes pressed Button on game table
* Input: appropriate pointer to Button, number of the button
* Output: Nothing
*******************************************************************************/
void ButtonClick(TButton *Butt, char No){
  // play move only if game is not finished
  if (victory == 0){
    if (MyTurn == 0){               // first move detected
      MyTurn = XO_Turn;
      NVIC_IntEnable(IVT_INT_TIMER3A_16_32_bit); // start to measure game duration
    }
    // if my turn
    if (XO_turn == MyTurn){
      // if valid move
      if (XO_Set_New_State(No)){
        // process my move
        XO_Process(No);
        // if RF is used send my move
        if (RF_used){
          LED_G = 0;
          LED_R = 1;
          // send my move
          RF_Send_Move();
          Delay_ms(50);
        }
        else{
          // if RF is not used, it is still my move
          MyTurn = XO_Turn;
          if (XO_Turn == 1){
            LED_G = 1;
            LED_B = 0;
          }
          else{
            LED_B = 1;
            LED_G = 0;
          }
        }
        // test if the game is not completed
        Test_Victory();
        TestDraw();
        XO_First_Move ++;
      }
    }
  }
}

/*******************************************************************************
* End of File
*******************************************************************************/