#include "mikromedia_5_TIVA_objects.h"
#include "mikromedia_5_TIVA_resources.h"

//--------------------- User code ---------------------//
#include "Init_routines.h"
#include "ADXL345_main.h"
#include "HID_Mouse.h"
#include "sensors.h"
#include "Paint.h"
#include "RGB_Light.h"
#include "Power_management.h"
#include "Slide_Show.h"
#include "Tic_Tac_Toe.h"
#include "MP3_main.h"
#include "MP3_driver.h"
#include "Scroll_main.h"
#include "RTCC_main.h"
#include "WiFi_main.h"

// extern variables from mikromedia_5_TIVA_driver
extern char PenDown;
extern unsigned int Xcoord, Ycoord;

// static variables
static unsigned int Main_Example_counter = 50000;
static unsigned int Clock_Example_counter = 0;
static char Example_State;

// main state machine
void Run_Example(){
  switch (Example_State) {
    case 0 : {                       // Home screen
              if (Main_Example_counter++ > 30000){
                Main_Example_counter = 0;
                HID_Mouse_Restore_Display();
                checkRTC();
                checkPowerM();
                HID_Mouse_Restore_Buffer();
              }
             }; break;
    case 1 : {                       // Accel Screen
              doAccel();
             }; break;
    case 2 : {                       // RTCC screen
              if (Clock_Example_counter++ > 10000){
                Clock_Example_counter = 0;
                if ((PenDown == 0) && (My_PenDown == 0))
                  doRTC_Example();
              }
             }; break;
    case 3 : break;                  // RGB screen
    case 4 : {                       // Sensors screen
              Sensors_Read();
             }; break;
    case 5 : {                       // MP3
              doMp3();
             }; break;
    case 6 : break;                  // Paint
    case 7 : break;                  // Slide Show
    case 8 : {                       // Tic Tac Toe
               doTicTacToe();
               Delay_ms(5);
             }; break;
  }
}

// volume bar for mp3 example
void UpdateVolumeBar(char vol, char old_vol){
  unsigned int xcoo;

  xcoo = (vol*6)/5 + VolmeBar.Left;

  if (xcoo < VolmeBar.Left)
    xcoo = VolmeBar.Left;

  if (xcoo > VolmeBar.Left + VolmeBar.Width)
    xcoo = VolmeBar.Left + VolmeBar.Width;

  if (vol >= old_vol){
    TFTIntern_Set_Brush(1, 0x55FF, 0, 0, 0, 0);
    TFTIntern_Set_Pen(0x55FF, 0);
    TFTIntern_Rectangle(VolmeBar.Left, VolmeBar.Top, xcoo, VolmeBar.Top + VolmeBar.Height);
  }
  else{
    TFTIntern_Set_Brush(1, VolmeBar.Color, 0, 0, 0, 0);
    TFTIntern_Set_Pen(VolmeBar.Color, 0);
    TFTIntern_Rectangle(xcoo + 1, VolmeBar.Top, VolmeBar.Left + VolmeBar.Width + 1, VolmeBar.Top + VolmeBar.Height);
  }
}

//----------------- End of User code ------------------//

// Event Handlers

void Image2OnPress() {
  Example_State = 0;
  BLED_Fade_Out();
  HID_Mouse_Restore_Display();
  DrawScreen(&Home);
  HID_Mouse_Restore_Buffer();
  Display_Bat_Stat = 1;
  checkPowerM();
  BLED_Fade_In();
}

void Image1OnPress() {
  Example_State = 1;
  BLED_Fade_Out();
  HID_Mouse_Restore_Display();
  DrawScreen(&Accelerometer);
  HID_Mouse_Restore_Buffer();
  BLED_Fade_In();
  ACCEL_Start();
}

void Image3OnPress() {
  BLED_Fade_Out();
  
  if (RGB_Image_Loaded == 0){
    HID_Mouse_Restore_Display();
    DrawScreen(&Loading);
    HID_Mouse_Restore_Buffer();
    BLED_Fade_In();
    if (Init_FAT()){                                          // displays background picture
      CopyImage_FromResources_ToSRAM(rgb_bitmap_bmp, RGB_Color_Image_SRAM_ptr);
      RGB_Image_Loaded = 1;
    }
    BLED_Fade_Out();
  }
  HID_Mouse_Restore_Display();
  DrawScreen(&RGBLight);
  RGB_Init();
  HID_Mouse_Restore_Buffer();
  Example_State = 3;
  BLED_Fade_In();
}

void Image5OnPress() {
  BLED_Fade_Out();
  RGB_OFF();
  HID_Mouse_Restore_Display();
  DrawScreen(&Home);
  HID_Mouse_Restore_Buffer();
  Display_Bat_Stat = 1;
  checkPowerM();
  BLED_Fade_In();
  Example_state = 0;
}

void Image4OnPress() {
//  PickColor();
}

void Image6OnClick() {
  BLED_Fade_Out();
  HID_Mouse_Restore_Display();
  DrawScreen(&Clock);
  HID_Mouse_Restore_Buffer();
  BLED_Fade_In();
  RTC_Example_init();
  Example_State = 2;
}

void Box2OnPress() {
  HoursTens();
}

void Box3OnPress() {
  HoursOnes();
}

void Box4OnPress() {
  MinutesTens();
}

void Box5OnPress() {
  MinutesOnes();
}

void ClockBackOnPress() {
  BLED_Fade_Out();
  HID_Mouse_Restore_Display();
  DrawScreen(&Home);
  HID_Mouse_Restore_Buffer();
  Display_Bat_Stat = 1;
  checkPowerM();
  BLED_Fade_In();
  Example_State = 0;
}

void Image8OnPress() {
  if (Set_MyRTCC() == 0)
    return;
  Example_State = 0;
  BLED_Fade_Out();
  HID_Mouse_Restore_Display();
  DrawScreen(&Home);
  HID_Mouse_Restore_Buffer();
  Display_Bat_Stat = 1;
  checkPowerM();
  BLED_Fade_In();
}

void BackIconSensorsOnPress() {
  Sensors_Stop();
  HID_Mouse_Restore_Display();
  DrawScreen(&Home);
  HID_Mouse_Restore_Buffer();
  Display_Bat_Stat = 1;
  checkPowerM();
  Example_state = 0;
  BLED_Fade_In();
}

void Image9OnPress() {
  Sensors_Init();
  Example_state = 4;
  BLED_Fade_Out();
  HID_Mouse_Restore_Display();
  DrawScreen(&Sensors);
  HID_Mouse_Restore_Buffer();
  Sensors_Read();
  BLED_Fade_In();
}

void Image16OnClick() {
char i, temp_text[20], txt[2];
unsigned int *Image_Add[2];
unsigned long Images[2];

  Image_Add[0] = MP3_Image1_SRAM_ptr;
  Image_Add[1] = MP3_Image2_SRAM_ptr;
  
  Images[0] = mp3_note_bmp;
  Images[1] = mp3_player_note_bmp;
  
  if (MP3_Image_Loaded == 0){
    BLED_Fade_Out();
    DrawScreen(&Loading);
    BLED_Fade_In();

    for (i = 0; i < 2; i ++){
      if (Init_FAT()){                            // copy background picture
        temp_text[0] = 0;
        strcpy(temp_text, Label46_Caption);
        strcat(temp_text, "  Image ");
        txt[0] = i + 1 + 48;
        txt[2] = 0;
        strcat(temp_text, txt);
        Label46.Caption = temp_text;
        RedrawLabel(Loading.COlor, &Label46, 200, 35);

        CopyImage_FromResources_ToSRAM(Images[i], Image_Add[i]);
      }
    }
    MP3_Image_Loaded = 1;
  }
  Label46.Caption = Label46_Caption;

  BLED_Fade_Out();
  
  HID_Mouse_Restore_Display();
  DrawScreen(&MP3_SongList);
  HID_Mouse_Restore_Buffer();
  Mp3_Init();
  Example_state = 5;
  BLED_Fade_In();
}

void mp3_back_iconOnPress() {
  BLED_Fade_Out();
  HID_Mouse_Restore_Display();
  DrawScreen(&Home);
  HID_Mouse_Restore_Buffer();
  Display_Bat_Stat = 1;
  checkPowerM();
  Example_state = 0;
  BLED_Fade_In();
}

void Label22OnPress() {

}

void Image15OnPress() {
  BLED_Fade_Out();
  HID_Mouse_Restore_Display();
  DrawScreen(&Home);
  HID_Mouse_Restore_Buffer();
  Display_Bat_Stat = 1;
  checkPowerM();
  Example_state = 0;
  Stop_Scroll();
  BLED_Fade_In();
}

void Label24OnPress() {

}

void Box6OnPress() {
  ScrollPress();
}

void Box6OnClick() {
  ScrollOnClick();
}

void Icon_back_mp3OnClick() {
  BLED_Fade_Out();
  HID_Mouse_Restore_Display();
  DrawScreen(&Home);
  HID_Mouse_Restore_Buffer();
  Display_Bat_Stat = 1;
  checkPowerM();
  Example_state = 0;
  BLED_Fade_In();
}

void Label27OnPress() {

}

/* PAINT CODE */
void DrawingAreaOnPress() {
  if ((Xcoord >= DrawingArea.Left+drawingTool.pen_size) && (Ycoord >= DrawingArea.Top+drawingTool.pen_size)){
    if (drawingTool.EraserSelected){
      TFTIntern_Set_Pen(DrawingArea.Color, 0);
      TFTIntern_Set_Brush(1, DrawingArea.Color, 0, 0, 0, 0);
    }
    HID_Mouse_Restore_Display();
    TFTIntern_Circle(Xcoord, Ycoord, drawingTool.pen_size);
    HID_Mouse_Restore_Buffer();
  }
}

void Color01OnPress() {
  SelectColor(&Color01);
}

void Color02OnPress() {
  SelectColor(&Color02);
}

void Color03OnPress() {
  SelectColor(&Color03);
}

void Color04OnPress() {
  SelectColor(&Color04);
}

void Color05OnPress() {
  SelectColor(&Color05);
}

void Color06OnPress() {
  SelectColor(&Color06);
}

void Color07OnPress() {
  SelectColor(&Color07);
}

void Color08OnPress() {
  SelectColor(&Color08);
}

void Color09OnPress() {
  SelectColor(&Color09);
}

void Color10OnPress() {
  SelectColor(&Color10);
}

void Color11OnPress() {
  SelectColor(&Color11);
}

void Color12OnPress() {
  SelectColor(&Color12);
}

void Color13OnPress() {
  SelectColor(&Color13);
}

void Color14OnPress() {
  SelectColor(&Color14);
}

void Color15OnPress() {
  SelectColor(&Color15);
}

void Color16OnPress() {
  SelectColor(&Color16);
}

void Color17OnPress() {
  SelectColor(&Color17);
}

void Color18OnPress() {
  SelectColor(&Color18);
}

void Color19OnPress() {
  SelectColor(&Color19);
}

void Color20OnPress() {
  SelectColor(&Color20);
}

void Color21OnPress() {
  SelectColor(&Color21);
}

void Color22OnPress() {
  SelectColor(&Color22);
}

void Color23OnPress() {
  SelectColor(&Color23);
}

void Color24OnPress() {
  SelectColor(&Color24);
}

void Color25OnPress() {
  SelectColor(&Color25);
}

void Color26OnPress() {
  SelectColor(&Color26);
}

void Color27OnPress() {
  SelectColor(&Color27);
}

void Color28OnPress() {
  SelectColor(&Color28);
}

void Color29OnPress() {
  SelectColor(&Color29);
}

void Color30OnPress() {
  SelectColor(&Color30);
}

void Pen1OnPress() {
  if (drawingTool.CurrentPen != &Pen1){
    drawingTool.pen_size = 2;
    drawingTool.PreviousPen = drawingTool.CurrentPen;
    drawingTool.CurrentPen = &Pen1;
    
    HID_Mouse_Restore_Display();
    UpdatePenSelection();
    HID_Mouse_Restore_Buffer();
  }
}

void Pen2OnPress() {
  if (drawingTool.CurrentPen != &Pen2){
    drawingTool.pen_size = 3;
    drawingTool.PreviousPen = drawingTool.CurrentPen;
    drawingTool.CurrentPen = &Pen2;
    
    HID_Mouse_Restore_Display();
    UpdatePenSelection();
    HID_Mouse_Restore_Buffer();
  }
}

void Pen3OnPress() {
  if (drawingTool.CurrentPen != &Pen3){
    drawingTool.pen_size = 5;
    drawingTool.PreviousPen = drawingTool.CurrentPen;
    drawingTool.CurrentPen = &Pen3;

    HID_Mouse_Restore_Display();
    UpdatePenSelection();
    HID_Mouse_Restore_Buffer();
  }
}

void NewScreenOnPress() {
  HID_Mouse_Restore_Display();

  DrawBox(&DrawingArea);
  UpdatePenColor();
  
  HID_Mouse_Restore_Buffer();
}

void BackGroundColorOnPress() {
  HID_Mouse_Restore_Display();

  DrawingArea.Color = drawingTool.CurrentColor->Color;
  DrawBox(&DrawingArea);

  HID_Mouse_Restore_Buffer();
}

void EraserOnPress() {
  if (drawingTool.EraserSelected != 1){
    HID_Mouse_Restore_Display();
  
    Pen.Picture_Name = icon_brush_bmp;
    DrawImage(&Pen);
    Eraser.Picture_Name = icon_eraser_selected_bmp;
    DrawImage(&Eraser);
    drawingTool.EraserSelected = 1;
    UpdatePenColor();
    
    HID_Mouse_Restore_Buffer();
  }
}

void PenOnPress() {
  if (drawingTool.EraserSelected != 0){
    drawingTool.EraserSelected = 0;
    Eraser.Picture_Name = icon_eraser_bmp;
    
    HID_Mouse_Restore_Display();
    DrawImage(&Eraser);
    Pen.Picture_Name = icon_brush_selected_bmp;
    DrawImage(&Pen);
    UpdatePenColor();
    
    HID_Mouse_Restore_Buffer();
  }
}

void Image24OnPress() {
  BLED_Fade_Out();
  SetDefaults();
  HID_Mouse_Restore_Display();
  DrawScreen(&Paint);
  HID_Mouse_Restore_Buffer();
  Example_state = 6;
  BLED_Fade_In();  
}

void icon_paint_backOnPress() {
  BLED_Fade_Out();
  HID_Mouse_Restore_Display();
  DrawScreen(&Home);
  HID_Mouse_Restore_Buffer();
  Display_Bat_Stat = 1;
  checkPowerM();
  Example_state = 0;
  BLED_Fade_In();
}

void pause_iconOnPress() {
  HID_Mouse_Restore_Display();

  MP3_Example_State = 3;
  pause_icon.Active = 0;
  Play_icon.Active = 1;
  DrawImage(&play_icon);
  
  HID_Mouse_Restore_Buffer();
}

void songlist_iconOnPress() {
  HID_Mouse_Restore_Display();

  StopMP3_File();
//  Mp3_Init();
  Example_state = 5;
  MP3_Example_State = 0;
  
  HID_Mouse_Restore_Buffer();
}
  
void stop_iconOnDown() {
  HID_Mouse_Restore_Display();

  StopMP3_File();
  MP3_Example_State = 1;
  pause_icon.Active = 0;
  Play_icon.Active = 1;
  DrawImage(&stop_icon);
  DrawImage(&play_icon);
  
  HID_Mouse_Restore_Buffer();
}

void next_iconOnDown() {
  HID_Mouse_Restore_Display();

  GetNext_mp3_File(1);
  DrawImage(&next_icon);
  pause_icon.Active = 1;
  Play_icon.Active = 0;
  DrawImage(&pause_icon);
  
  HID_Mouse_Restore_Buffer();
}

void previous_iconOnDown() {
  HID_Mouse_Restore_Display();

  GetNext_mp3_File(0);
  DrawImage(&previous_icon);
  pause_icon.Active = 1;
  Play_icon.Active = 0;
  DrawImage(&pause_icon);
  
  HID_Mouse_Restore_Buffer();
}
  
void VolmeBarOnPress() {
  unsigned int temp;

  temp = Xcoord - VolmeBar.Left;
  temp = temp * 100;
  level = temp / VolmeBar.Width;
  MP3_Set_Volume(100 - level, 100 - level);

  HID_Mouse_Restore_Display();
  UpdateVolumeBar(level, old_level);
  HID_Mouse_Restore_Buffer();
  old_level = level;
}

void ProgreesBarOnPress() {

}

void Play_iconOnDown() {
  HID_Mouse_Restore_Display();

  pause_icon.Active = 1;
  Play_icon.Active = 0;
  if (MP3_Example_State == 1)
    MP3_play(Label28_Caption, current_file);
  MP3_Example_State = 2;
  DrawImage(&pause_icon);
  
  HID_Mouse_Restore_Buffer();
}

void Icon_next_boxOnPress() {
  HID_Mouse_Restore_Display();

  Slide_Index ++;
  if (Slide_Index > 5)
    Slide_Index = 0;

  Slide_DIsplaySlide(Slide_Index);

  ByteToStr(Slide_Index + 1, Label35_caption);
  RedrawLabel(SlideShow.Color, &Label35, 25, 30);
  
  HID_Mouse_Restore_Buffer();
}

void icon_previous_boxOnPress() {
  HID_Mouse_Restore_Display();
  
  Slide_Index --;
  if (Slide_Index < 0)
    Slide_Index = 5;
    
  Slide_DIsplaySlide(Slide_Index);
  
  ByteToStr(Slide_Index + 1, Label35_caption);
  RedrawLabel(SlideShow.Color, &Label35, 25, 30);
  
  HID_Mouse_Restore_Buffer();
}

void Image18OnClick() {
char i, temp_text[20], txt[2];

  if (SlideShow_Image_Loaded == 0){
    BLED_Fade_Out();
    HID_Mouse_Restore_Display();
    DrawScreen(&Loading);
    HID_Mouse_Restore_Buffer();
    BLED_Fade_In();
    
    for (i = 0; i < 6; i ++){
      if (Init_FAT()){                            // copy background picture
        temp_text[0] = 0;
        strcpy(temp_text, Label46_Caption);
        strcat(temp_text, "  Slide ");
        txt[0] = i + 1 + 48;
        txt[2] = 0;
        strcat(temp_text, txt);
        Label46.Caption = temp_text;
        RedrawLabel(Loading.COlor, &Label46, 200, 35);
        
        CopyImage_FromResources_ToSRAM(Slides[i], Slides_Address[i]);
      }
    }
    SlideShow_Image_Loaded = 1;
  }
  Label46.Caption = Label46_Caption;
  
  BLED_Fade_Out();
  HID_Mouse_Restore_Display();
  DrawScreen(&SlideShow);
  HID_Mouse_Restore_Buffer();
  Start_Slide_Show();
  Example_state = 7;
  ByteToStr(Slide_Index + 1, Label35_caption);
  RedrawLabel(SlideShow.Color, &Label35, 20, 30);
  BLED_Fade_In();
}

void Box7OnDown() {
  BLED_Fade_Out();
  HID_Mouse_Restore_Display();
  DrawScreen(&Home);
  HID_Mouse_Restore_Buffer();
  Display_Bat_Stat = 1;
  checkPowerM();
  Example_state = 0;
  Stop_Scroll();
  BLED_Fade_In();
}

void RGB_Active_areaOnPress() {
  PickColor();
}

void Box9OnPress() {
  BLED_Fade_Out();
  HID_Mouse_Restore_Display();
  DrawScreen(&Home);
  HID_Mouse_Restore_Buffer();
  Display_Bat_Stat = 1;
  checkPowerM();
  Example_state = 0;
//  NVIC_IntDisable(IVT_INT_TIM5);
  BLED_Fade_In();
  LED_R = 0;
  LED_B = 0;
  LED_G = 0;
}

void Box11OnPress() {
  XO_Reset();
}

void Button1OnClick() {
  ButtonClick(ButtonArray[0], 0);
}

void Button2OnClick() {
  ButtonClick(ButtonArray[1], 1);
}

void Button3OnClick() {
  ButtonClick(ButtonArray[2], 2);
}

void Button4OnClick() {
  ButtonClick(ButtonArray[3], 3);
}

void Button5OnClick() {
  ButtonClick(ButtonArray[4], 4);
}

void Button6OnClick() {
  ButtonClick(ButtonArray[5], 5);
}

void Button7OnClick() {
  ButtonClick(ButtonArray[6], 6);
}

void Button8OnClick() {
  ButtonClick(ButtonArray[7], 7);
}

void Button9OnClick() {
  ButtonClick(ButtonArray[8], 8);
}

void Image22OnClick() {
  BLED_Fade_Out();

  if (TicTacToe_Table_Loaded == 0){
    HID_Mouse_Restore_Display();
    DrawScreen(&Loading);
    HID_Mouse_Restore_Buffer();
    BLED_Fade_In();
    if (Init_FAT()){                                          // displays background picture
      CopyImage_FromResources_ToSRAM(game_table_bmp, TicTacToe_Table_Image_SRAM_ptr);
      TicTacToe_Table_Loaded = 1;
    }
    BLED_Fade_Out();
  }
  
  HID_Mouse_Restore_Display();
  DrawScreen(&Tic_Tac_Toe);
  HID_Mouse_Restore_Buffer();
  Example_state = 8;
  // init RGB
  Init_FAT();
  XO_Reset();
  BLED_Fade_In();
}

void Box12OnPress() {

}

void Slide_Active_areaOnPress() {
//  Slide_scroll();
}

void CheckBox1OnClick() {
  Start_nRF();
}

void Box10OnClick() {
  if (CheckBox1.Checked == 0)
    CheckBox1.Checked = 1;
  else
    CheckBox1.Checked = 0;
  DrawCheckBox(&CheckBox1);
  Start_nRF();
}


void Box12OnClick() {
  Deactivate_All_Rollers();
  Start_Ethernet();
}

void Box13OnClick() {
  Deactivate_All_Rollers();
}

void Box14OnPress() {
  AmPmRun();
}